#!/usr/bin/env python
# -- Content-Encoding: UTF-8 --
"""
iPOPO remote shell

Provides a remote interface for the Pelix shell that can be accessed using
telnet or netcat.

:author: Thomas Calmant
:copyright: Copyright 2013, isandlaTech
:license: GPLv3
:version: 0.1.1
:status: Alpha

..

    This file is part of iPOPO.

    iPOPO is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    iPOPO is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with iPOPO. If not, see <http://www.gnu.org/licenses/>.
"""

# Module version
__version_info__ = (0, 1, 1)
__version__ = ".".join(map(str, __version_info__))

# Documentation strings format
__docformat__ = "restructuredtext en"

# ------------------------------------------------------------------------------

from pelix.ipopo.decorators import ComponentFactory, Requires, Property, \
    Validate, Invalidate, Provides
from pelix.shell import SHELL_SERVICE_SPEC, REMOTE_SHELL_SPEC

# ------------------------------------------------------------------------------

from select import select

import logging
import sys
import threading
import socket

if sys.version_info[0] >= 3:
    import socketserver
else:
    import SocketServer as socketserver

# ------------------------------------------------------------------------------

_logger = logging.getLogger(__name__)

# ------------------------------------------------------------------------------

class SharedBoolean(object):
    """
    Shared boolean between objects / threads
    """
    def __init__(self, value=False):
        """
        Set up members
        """
        self._lock = threading.Lock()
        self._value = value


    def get_value(self):
        """
        Retrieves the boolean value
        """
        with self._lock:
            return self._value


    def set_value(self, value):
        """
        Sets the boolean value
        """
        with self._lock:
            self._value = value

# ------------------------------------------------------------------------------

class RemoteConsole(socketserver.StreamRequestHandler):
    """
    Handles incoming connections and redirect network stream to the Pelix shell
    """
    def __init__(self, shell_svc, active_flag, *args):
        """
        Sets up members
        
        :param shell_svc: The underlying Pelix shell service
        :param active_flag: Common flag for stopping the client communication
        """
        self._shell = shell_svc
        self._active = active_flag
        socketserver.StreamRequestHandler.__init__(self, *args)


    def send(self, data):
        """
        Tries to send data to the client.
        
        :param data: Data to be sent
        :return: True if the data was sent, False on error
        """
        if data is not None:
            data = data.encode("UTF-8")

        try:
            self.wfile.write(data)
            self.wfile.flush()
            return True

        except IOError:
            # An error occurred, mask it
            # -> This allows to handle the command even if the client has been
            # disconnect (i.e. "echo stop 0 | nc localhost 9000")
            return False


    def handle(self):
        """
        Handles a TCP client
        """
        _logger.info("RemoteConsole client connected: [%s]:%d",
                     self.client_address[0], self.client_address[1])

        # Print the banner
        ps1 = self._shell.get_ps1()
        self.send(self._shell.get_banner())
        self.send(ps1)

        try:
            while self._active.get_value():
                # Wait for data
                rlist = select([self.rfile], [], [], .5)[0]
                if not rlist:
                    # Nothing to do (poll timed out)
                    continue

                data = self.rfile.readline()
                if not data:
                    # End of stream (client gone)
                    break

                # Strip the line
                line = data.strip()
                if not data:
                    # Empty line
                    continue

                # Execute it
                try:
                    self._shell.handle_line(self.rfile, self.wfile, line)

                except KeyboardInterrupt:
                    # Stop there on interruption
                    self.send("\nInterruption received.")
                    return

                except IOError as ex:
                    # I/O errors are fatal
                    _logger.exception("Error communicating with a client: %s",
                                      ex)
                    break

                except Exception as ex:
                    # Other exceptions are not important
                    import traceback
                    self.send("\nError during last command: %s\n" % ex)
                    self.send(traceback.format_exc())

                # Print the prompt
                self.send(ps1)

        finally:
            _logger.info("RemoteConsole client gone: [%s]:%d",
                         self.client_address[0], self.client_address[1])

            # Be polite
            self.send("\nSession closed. Good bye.\n")

# ------------------------------------------------------------------------------

class ThreadingTCPServerFamily(socketserver.ThreadingTCPServer):
    """
    Threaded TCP Server handling different address families
    """
    def __init__(self, server_address, request_handler_class):
        """
        Sets up the TCP server. Doesn't bind nor activate it.
        """
        # Determine the address family
        addr_info = socket.getaddrinfo(server_address[0], server_address[1],
                                       0, 0, socket.SOL_TCP)

        # Change the address family before the socket is created
        # Get the family of the first possibility
        self.address_family = addr_info[0][0]

        # Call the super constructor
        socketserver.ThreadingTCPServer.__init__(self, server_address,
                                                 request_handler_class,
                                                 False)

        if self.address_family == socket.AF_INET6:
            # Explicitly ask to be accessible both by IPv4 and IPv6
            # Some versions of Python don't have V6ONLY.
            # On Linux, IPC6_V6ONLY = 26
            IPV6_V6ONLY = getattr(socket, "IPV6_V6ONLY", 26)

            try:
                self.socket.setsockopt(socket.IPPROTO_IPV6, IPV6_V6ONLY, 0)

            except socket.error as ex:
                # Log the error
                _logger.exception("Couldn't set IP double stack flag: %s", ex)


def _create_server(shell, server_address, port):
    """
    Creates the TCP console on the given address and port
    
    :param shell: The remote shell handler
    :param server_address: Server bound address
    :param port: Server port
    :return: server thread, TCP server object
    """
    # Set up the request handler creator
    active_flag = SharedBoolean(True)
    request_handler = lambda *args: RemoteConsole(shell, active_flag, *args)

    # Set up the server
    server = ThreadingTCPServerFamily((server_address, port), request_handler)

    # Set flags
    server.daemon_threads = True
    server.allow_reuse_address = True

    # Activate the server
    server.server_bind()
    server.server_activate()

    # Serve clients
    server_thread = threading.Thread(target=server.serve_forever)
    server_thread.daemon = True
    server_thread.start()

    return (server_thread, server, active_flag)

# ------------------------------------------------------------------------------

@ComponentFactory("ipopo-remote-shell-factory")
@Provides(REMOTE_SHELL_SPEC)
@Property("_address", "pelix.shell.address", "localhost")
@Property("_port", "pelix.shell.port", 9000)
@Requires("_shell", SHELL_SERVICE_SPEC)
class IPopoRemoteShell(object):
    """
    The iPOPO Remote Shell, based on the Pelix Shell
    """
    def __init__(self):
        """
        Sets up the component
        """
        # Component shell
        self._shell = None
        self._address = None
        self._port = 0

        # Internals
        self._thread = None
        self._server = None
        self._server_flag = None


    def get_access(self):
        """
        Implementation of the remote shell specification
        
        :return: A (host, port) tuple
        """
        return (self._address, self._port)


    def get_banner(self):
        """
        Retrieves the shell banner
        
        :return: The shell banner
        """
        line = '-' * 72
        shell_banner = self._shell.get_banner()

        return "{lines}\n{shell_banner}\niPOPO Remote Shell\n{lines}\n" \
            .format(lines=line, shell_banner=shell_banner)


    def get_ps1(self):
        """
        Returns the shell prompt
        
        :return: The shell prompt
        """
        return self._shell.get_ps1()


    def handle_line(self, rfile, wfile, line):
        """
        Handles the command line.
        
        **Does not catch exceptions !**
        
        :param rfile: Input file-like object
        :param wfile: Output file-like object
        :param line: The command line
        :return: The execution result (True on success, else False)
        """
        return self._shell.execute(line, rfile, wfile)


    @Validate
    def validate(self, context):
        """
        Component validation
        """
        if not self._address:
            # Local host by default
            self._address = "localhost"

        try:
            self._port = int(self._port)
            if self._port < 0 or self._port > 65535:
                # Invalid port value
                self._port = 0

        except ValueError:
            # Invalid port string: use a random port
            self._port = 0

        # Start the TCP server
        self._thread, self._server, self._server_flag = \
                                _create_server(self, self._address, self._port)

        # Property update (if port was 0)
        self._port = self._server.socket.getsockname()[1]

        _logger.info("RemoteShell validated on port: %d", self._port)


    @Invalidate
    def invalidate(self, context):
        """
        Component invalidation
        """
        # Stop the clients loops
        self._server_flag.set_value(False)

        # Shutdown the server
        self._server.shutdown()
        self._thread.join(2)

        # Close the server socket (ignore errors)
        self._server.server_close()

        # Clean up
        self._thread = None
        self._server = None
        self._server_flag = None

        _logger.info("RemoteShell gone from port: %d", self._port)




#!/usr/bin/env python
# -- Content-Encoding: UTF-8 --
"""
Dependency-less LDAP filter parser for Python

:author: Thomas Calmant
:copyright: Copyright 2013, isandlaTech
:license: GPLv3
:version: 0.3.1
:status: Alpha

..

    This file is part of iPOPO.

    iPOPO is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    iPOPO is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with iPOPO. If not, see <http://www.gnu.org/licenses/>.
"""

from pelix.utilities import is_string
import inspect

# ------------------------------------------------------------------------------

# Module version
__version_info__ = (0, 3, 1)
__version__ = ".".join(map(str, __version_info__))

# Documentation strings format
__docformat__ = "restructuredtext en"

# ------------------------------------------------------------------------------

ESCAPE_CHARACTER = '\\'
""" The LDAP escape character: \\"""

# Do not insert the ESCAPE CHARACTER in this list
ESCAPED_CHARACTERS = "()&|=<>~*+#,;'\""
"""
The characters to escape in an LDAP string.
See http://www.ldapexplorer.com/en/manual/109010000-ldap-filter-syntax.htm
"""

# ------------------------------------------------------------------------------

AND = 0
""" 'And' LDAP operation """

OR = 1
""" 'Or' LDAP operation """

NOT = 2
""" 'Not' LDAP operation """

# ------------------------------------------------------------------------------

class LDAPFilter(object):
    """
    Represents an LDAP filter
    """
    def __init__(self, operator):
        """
        Initializer
        """
        if operator not in (AND, OR, NOT):
            raise ValueError("Invalid operator: {0}".format(operator))

        self.subfilters = []
        self.operator = operator


    def __eq__(self, other):
        """
        Equality testing
        """
        if type(other) is not LDAPFilter:
            # Bad type
            return False

        if self.operator != other.operator:
            # Different operators
            return False

        # All sub-filters must match
        if len(self.subfilters) != len(other.subfilters):
            # Not the same size...
            return False

        for subfilter in self.subfilters:
            if subfilter not in other.subfilters:
                # Missing sub filter
                return False

        # Same content
        return True


    def __ne__(self, other):
        """
        Inequality testing
        """
        return not self.__eq__(other)


    def __repr__(self):
        """
        String description
        """
        return "{0}.get_ldap_filter({1!r})".format(__name__, self.__str__())


    def __str__(self):
        """
        String representation
        """
        return "({0}{1})".format(operator2str(self.operator),
               "".join([str(subfilter) for subfilter in self.subfilters]))


    def append(self, ldap_filter):
        """
        Appends a filter or a criteria to this filter

        :param ldap_filter: An LDAP filter or criteria
        :raise TypeError: If the parameter is not of a known type
        :raise ValueError: If the more than one filter is associated to a
                           NOT operator
        """
        if not isinstance(ldap_filter, (LDAPFilter, LDAPCriteria)):
            raise TypeError("Invalid filter type: {0}".format(
                                                type(ldap_filter).__name__))

        if len(self.subfilters) >= 1 and self.operator == NOT:
            raise ValueError("Not operator only handles one child")

        self.subfilters.append(ldap_filter)


    def matches(self, properties):
        """
        Tests if the given properties matches this LDAP filter and its children

        :param properties: A dictionary of properties
        :return: True if the properties matches this filter, else False
        """
        result = False

        for criteria in self.subfilters:
            if not criteria.matches(properties):
                if self.operator == AND:
                    # A criteria doesn't match in an "AND" test : short cut
                    result = False
                    break

            else:
                result = True
                if self.operator == OR:
                    # At least one match in a "OR" test : short cut
                    break

        if self.operator == NOT:
            # Revert result
            return not result

        return result


    def normalize(self):
        """
        Returns the first meaningful object in this filter.
        """
        if not self.subfilters:
            # No sub-filters
            return None

        # New sub-filters list
        new_filters = []
        for subfilter in self.subfilters:
            # Normalize the sub-filter before storing it
            norm_filter = subfilter.normalize()
            if norm_filter is not None and norm_filter not in new_filters:
                new_filters.append(norm_filter)

        # Update the instance
        self.subfilters = new_filters

        size = len(self.subfilters)
        if size > 1:
            # Normal filter
            return self

        else:
            if self.operator == NOT:
                # NOT is the only operator to accept 1 operand
                return self

            else:
                # Return the only child as the filter object
                return self.subfilters[0].normalize()


class LDAPCriteria(object):
    """
    Represents an LDAP criteria
    """
    def __init__(self, name, value, comparator):
        """
        Sets up the criteria

        :raise ValueError: If one of the parameters is empty
        """
        if not name or not value or not comparator:
            # Refuse empty values
            raise ValueError("Invalid criteria parameter ({0}, {1}, {2})"\
                             .format(name, value, comparator))

        if not inspect.isroutine(comparator):
            # Ensure we have a valid comparator
            raise ValueError("Comparator must be a method: {0}", comparator)

        self.name = str(name)
        self.value = value
        self.comparator = comparator


    def __eq__(self, other):
        """
        Equality testing
        """
        if type(other) is not LDAPCriteria:
            # Bad type
            return False

        for member in ('name', 'comparator'):
            if getattr(self, member) != getattr(other, member):
                # Difference found
                return False

        if type(self.value) == type(other.value):
            # Same type: direct comparison
            result = self.value == other.value
            return result

        else:
            # Convert to strings for comparison
            return str(self.value) == str(other.value)


    def __ne__(self, other):
        """
        Inequality testing
        """
        return not self.__eq__(other)


    def __repr__(self):
        """
        String representation
        """
        return '{0}.get_ldap_filter({1!r})'.format(__name__, self.__str__())


    def __str__(self):
        """
        String description
        """
        return "({0}{1}{2})".format(escape_LDAP(self.name),
                                    comparator2str(self.comparator),
                                    escape_LDAP(str(self.value)))


    def matches(self, properties):
        """
        Tests if the given criteria matches this LDAP criteria

        :param properties: A dictionary of properties
        :return: True if the properties matches this criteria, else False
        """
        if self.name not in properties:
            # Property is not even is the property
            return False

        # Use the comparator
        return self.comparator(self.value, properties[self.name])


    def normalize(self):
        """
        Returns this criteria
        """
        return self

# ------------------------------------------------------------------------------

def comparator2str(comparator):
    """
    Converts an operator method to a string
    """
    if comparator == _comparator_approximate:
        return "~="

    elif comparator == _comparator_eq:
        return "="

    elif comparator == _comparator_le:
        return "<="

    elif comparator == _comparator_lt:
        return "<"

    elif comparator == _comparator_ge:
        return ">="

    elif comparator == _comparator_gt:
        return ">"

    return "??"


def operator2str(operator):
    """
    Converts an operator value to a string
    """
    if operator == AND:
        return '&'

    elif operator == OR:
        return '|'

    elif operator == NOT:
        return '!'

    return '<unknown>'


def escape_LDAP(ldap_string):
    """
    Escape a string to let it go in an LDAP filter

    :param ldap_string: The string to escape
    :return: The protected string
    """
    if ldap_string is None:
        return None

    assert is_string(ldap_string)

    if len(ldap_string) == 0:
        # No content
        return ldap_string

    # Protect escape character previously in the string
    ldap_string = ldap_string.replace(ESCAPE_CHARACTER, \
                                      ESCAPE_CHARACTER + ESCAPE_CHARACTER)

    # Leading space
    if ldap_string.startswith(" "):
        ldap_string = "\\ {0}".format(ldap_string[1:])

    # Trailing space
    if ldap_string.endswith(" "):
        ldap_string = "{0}\\ ".format(ldap_string[:-1])

    # Escape other characters
    for escaped in ESCAPED_CHARACTERS:
        ldap_string = ldap_string.replace(escaped, ESCAPE_CHARACTER + escaped)

    return ldap_string


def unescape_LDAP(ldap_string):
    """
    Unespaces an LDAP string

    :param ldap_string: The string to unescape
    :return: The unprotected string
    """
    if ldap_string is None:
        return None

    assert is_string(ldap_string)

    if ESCAPE_CHARACTER not in ldap_string:
        # No need to loop
        return ldap_string

    escaped = False
    result = ""

    for character in ldap_string:
        if not escaped and character == ESCAPE_CHARACTER:
            # Escape character found
            escaped = True

        else:
            # Copy the character
            escaped = False
            result += character

    return result

# ------------------------------------------------------------------------------

def _comparator_star(filter_value, tested_value):
    """
    Tests a filter with a joker
    """
    if filter_value == '*':
        # The filter value is a joker : simple presence test
        if tested_value is None:
            return False

        elif hasattr(tested_value, "__len__"):
            # Refuse empty values
            return len(tested_value) != 0

        else:
            # Presence validated
            return True

    elif not is_string(tested_value):
        # Unhandled value type...
        return False

    parts = filter_value.split('*')

    i = 0
    last_part = len(parts) - 1

    idx = 0
    for part in parts:
        # Find the part in the tested value
        idx = tested_value.find(part, idx)

        if idx == -1:
            # Part not found
            return False

        if i == 0 and len(part) != 0 and idx != 0:
            # First part is not a star, but the tested value is not at
            # position 0 => Doesn't match
            return False

        if i == last_part and len(part) != 0 \
        and idx != len(tested_value) - len(part):
            # Last tested part is not at the end of the sequence
            return False

        # Be sure to test the next part
        idx += len(part)
        i += 1

    # Whole test passed
    return True


def _comparator_eq(filter_value, tested_value):
    """
    Tests if the filter value is equal to the tested value
    """
    if isinstance(tested_value, (list, tuple)):
        # Special case : lists
        if '*' in filter_value:
            # Special case : jokers (stars) in the filter
            for value in tested_value:
                if _comparator_star(filter_value, value):
                    # One match
                    return True

        # Convert the list items to strings
        for value in tested_value:

            # Try with the string conversion
            if not is_string(value):
                value_str = repr(value)
            else:
                value_str = value

            if filter_value == value_str:
                # Match !
                return True

        # Value not found
        return False

    elif '*' in filter_value:
        # Special case : jokers (stars) in the filter
        return _comparator_star(filter_value, tested_value)

    # Standard comparison
    if not is_string(tested_value):
        return filter_value == repr(tested_value)

    return filter_value == tested_value


def _comparator_approximate(filter_value, tested_value):
    """
    Tests if the filter value is nearly equal to the tested value.

    If the tested value is a string or an array of string, it compares their
    lower case forms
    """
    lower_filter_value = filter_value.lower()

    if is_string(tested_value):
        # Lower case comparison
        return _comparator_eq(lower_filter_value, tested_value.lower())

    elif hasattr(tested_value, '__iter__'):
        # Extract a list of strings
        new_tested = [value.lower() for value in tested_value \
                      if is_string(value)]

        if _comparator_eq(lower_filter_value, new_tested):
            # Value found in the strings
            return True

    # Compare the raw values
    return _comparator_eq(filter_value, tested_value) \
        or _comparator_eq(lower_filter_value, tested_value)


def _comparator_le(filter_value, tested_value):
    """
    Tests if the filter value is greater than the tested value

    tested_value <= filter_value
    """
    return _comparator_lt(filter_value, tested_value) \
        or _comparator_eq(filter_value, tested_value)


def _comparator_lt(filter_value, tested_value):
    """
    Tests if the filter value is strictly greater than the tested value

    tested_value < filter_value
    """
    if is_string(filter_value):
        value_type = type(tested_value)
        try:
            # Try a conversion
            filter_value = value_type(filter_value)

        except (TypeError, ValueError):
            if value_type is int:
                # Integer/float comparison trick
                try:
                    filter_value = float(filter_value)

                except (TypeError, ValueError):
                    # None-float value
                    return False

            else:
                # Incompatible type
                return False

    try:
        return tested_value < filter_value

    except TypeError:
        # Incompatible type
        return False


def _comparator_ge(filter_value, tested_value):
    """
    Tests if the filter value is lesser than the tested value

    tested_value >= filter_value
    """
    return _comparator_gt(filter_value, tested_value) \
        or _comparator_eq(filter_value, tested_value)


def _comparator_gt(filter_value, tested_value):
    """
    Tests if the filter value is strictly lesser than the tested value

    tested_value > filter_value
    """
    if is_string(filter_value):
        value_type = type(tested_value)
        try:
            # Try a conversion
            filter_value = value_type(filter_value)

        except (TypeError, ValueError):
            if value_type is int:
                # Integer/float comparison trick
                try:
                    filter_value = float(filter_value)

                except (TypeError, ValueError):
                    # None-float value
                    return False

            else:
                # Incompatible type
                return False

    try:
        return tested_value > filter_value

    except TypeError:
        # Incompatible type
        return False


# ------------------------------------------------------------------------------

def _compute_comparator(string, idx):
    """
    Tries to compute the LDAP comparator at the given index

    Valid operators are :

    * = : equality
    * <= : less than
    * >= : greater than
    * ~= : approximate

    :param string: A LDAP filter string
    :param idx: An index in the given string
    :return: The corresponding operator, None if unknown
    """
    part1 = string[idx]
    if part1 == '=':
        # Equality
        return _comparator_eq

    elif len(string) < idx + 2:
        # Too short string
        return None

    elif string[idx + 1] != '=':
        # It's a "strict" operator
        if part1 == '<':
            # Strictly lesser
            return _comparator_lt

        elif part1 == '>':
            # Strictly greater
            return _comparator_gt

    else:
        if part1 == '<':
            # Less or equal
            return _comparator_le

        elif part1 == '>':
            # Greater or equal
            return _comparator_ge

        elif part1 == '~':
            # Approximate equality
            return _comparator_approximate


def _compute_operation(string, idx):
    """
    Tries to compute the LDAP operation at the given index

    Valid operations are :

    * & : AND
    * | : OR
    * ! : NOT

    :param string: A LDAP filter string
    :param idx: An index in the given string
    :return: The corresponding operator (AND, OR or NOT)
    """
    operator = string[idx]

    if operator == '&':
        return AND

    elif operator == '|':
        return OR

    elif operator == '!':
        return NOT

    return None


def _skip_spaces(string, idx):
    """
    Retrieves the next non-space character after idx index in the given string

    :param string: The string to look into
    :param idx: The base search index
    :return: The next non-space character index, -1 if not found
    """
    i = idx
    size = len(string)

    while i < size:
        if not string[i].isspace():
            return i

        i += 1

    return -1


def _parse_LDAP_criteria(ldap_filter, startidx=0, endidx= -1):
    """
    Parses an LDAP sub filter (criteria)

    :param ldap_filter: An LDAP filter string
    :param startidx: Sub-filter start index
    :param endidx: Sub-filter end index
    :return: The LDAP sub-filter
    :raise ValueError: Invalid sub-filter
    """
    comparators = "=<>~"

    if endidx < 0:
        endidx = len(ldap_filter)

    if startidx >= endidx or startidx < 0 or endidx > len(ldap_filter):
        raise ValueError("Invalid string range start={0}, end={1}" \
                         .format(startidx, endidx))

    # Get the comparator
    i = startidx
    escaped = False
    while i < endidx:
        if not escaped:
            if ldap_filter[i] == ESCAPE_CHARACTER:
                # Next character escaped
                escaped = True

            elif ldap_filter[i] in comparators:
                # Comparator found
                break

        else:
            # Escaped character ignored
            escaped = False

        i += 1

    else:
        # Comparator never found
        raise ValueError("Comparator not found in '{0}'" \
                         .format(ldap_filter[startidx:endidx]))

    if i == startidx:
        # Attribute name is missing
        raise ValueError("Attribute name is missing in '{0}'" \
                         .format(ldap_filter[startidx:endidx]))

    comparator = _compute_comparator(ldap_filter, i)
    if comparator is None:
        # Unknown comparator
        raise ValueError("Unknown comparator in '{0}' - {1}\nFilter : {2}" \
                         .format(ldap_filter[startidx:endidx], ldap_filter[i],
                                 ldap_filter))

    # The attribute name can be extracted directly
    attribute_name = ldap_filter[startidx:i].strip()

    # Find the end of the comparator
    i += 1
    if i >= endidx:
        raise ValueError("Compared value is missing in '{0}'" \
                         .format(ldap_filter[startidx:endidx]))

    while ldap_filter[i] in comparators:
        i += 1

    # Skip spaces
    i = _skip_spaces(ldap_filter, i)

    # Extract the value name
    value = ldap_filter[i:endidx].strip()

    return LDAPCriteria(unescape_LDAP(attribute_name), unescape_LDAP(value),
                        comparator)


def _parse_LDAP(ldap_filter):
    """
    Parses the given LDAP filter string

    :param ldap_filter: An LDAP filter string
    :return: An LDAPFilter object, None if the filter was empty
    :raise ValueError: The LDAP filter string is invalid
    """
    if ldap_filter is None:
        # Nothing to do
        return None

    assert is_string(ldap_filter)

    # Remove surrounding spaces
    ldap_filter = ldap_filter.strip()
    if not ldap_filter:
        # Empty string
        return None

    escaped = False
    filter_len = len(ldap_filter)
    root = None
    stack = []
    subfilter_stack = []

    idx = 0
    while idx < filter_len:
        if not escaped:
            if ldap_filter[idx] == '(':
                # Opening filter : get the operator
                idx = _skip_spaces(ldap_filter, idx + 1)
                if idx == -1:
                    raise ValueError("Missing filter operator: {0}" \
                                     .format(ldap_filter))

                operator = _compute_operation(ldap_filter, idx)
                if operator is not None:
                    # New sub-filter
                    stack.append(LDAPFilter(operator))

                else:
                    # Sub-filter content
                    subfilter_stack.append(idx)

            elif ldap_filter[idx] == ')':
                # Ending filter : store it in its parent

                if len(subfilter_stack) != 0:
                    # Criteria finished
                    startidx = subfilter_stack.pop()
                    criteria = _parse_LDAP_criteria(ldap_filter, startidx, idx)

                    if len(stack) != 0:
                        top = stack.pop()
                        top.append(criteria)
                        stack.append(top)
                    else:
                        # No parent : filter contains only one criteria
                        # Make a parent to stay homogeneous
                        root = LDAPFilter(AND)
                        root.append(criteria)

                elif len(stack) != 0:
                    # Sub filter finished
                    ended_filter = stack.pop()

                    if len(stack) != 0:
                        top = stack.pop()
                        top.append(ended_filter)
                        stack.append(top)

                    else:
                        # End of the parse
                        root = ended_filter

                else:
                    raise ValueError("Too many end of parenthesis:{0}: {1}" \
                                     .format(idx, ldap_filter[idx:]))

            elif ldap_filter[idx] == '\\':
                # Next character must be ignored
                escaped = True

        else:
            # Escaped character ignored
            escaped = False

        # Don't forget to increment...
        idx += 1

    # No root : invalid content
    if root is None:
        raise ValueError("Invalid filter string")

    # Return the root of the filter
    return root.normalize()


def get_ldap_filter(ldap_filter):
    """
    Retrieves the LDAP filter object corresponding to the given filter.
    Parses it the argument if it is an LDAPFilter instance

    :param ldap_filter: An LDAP filter (LDAPFilter or string)
    :return: The corresponding filter, can be None
    :raise ValueError: Invalid filter string found
    :raise TypeError: Unknown filter type
    """
    if ldap_filter is None:
        return None

    if isinstance(ldap_filter, (LDAPFilter, LDAPCriteria)):
        # No conversion needed
        return ldap_filter

    elif is_string(ldap_filter):
        # Parse the filter
        return _parse_LDAP(ldap_filter)

    # Unknown type
    raise TypeError("Unhandled filter type {0}"\
                    .format(type(ldap_filter).__name__))


def combine_filters(filters, operator=AND):
    """
    Combines two LDAP filters, which can be strings or LDAPFilter objects

    :param filters: Filters to combine
    :param operator: The operator for combination
    :return: The combined filter, can be None if all filters are None
    :raise ValueError: Invalid filter string found
    :raise TypeError: Unknown filter type
    """
    if not filters:
        return None

    if not hasattr(filters, '__iter__') or is_string(filters):
        raise TypeError("Filters argument must be iterable")

    # Remove None filters and convert others
    ldap_filters = []
    for sub_filter in filters:
        if sub_filter is None:
            # Ignore None filters
            continue

        ldap_filter = get_ldap_filter(sub_filter)
        if ldap_filter is not None:
            # Valid filter
            ldap_filters.append(ldap_filter)

    if len(ldap_filters) == 0:
        # Do nothing
        return None

    elif len(ldap_filters) == 1:
        # Only one filter, return it
        return ldap_filters[0]

    new_filter = LDAPFilter(operator)

    for sub_filter in ldap_filters:
        # Direct combination
        new_filter.append(sub_filter)

    return new_filter.normalize()



#!/usr/bin/env python
# -- Content-Encoding: UTF-8 --
"""
Pelix Utilities: Task pool

:author: Thomas Calmant
:copyright: Copyright 2013, isandlaTech
:license: GPLv3
:version: 0.5.0
:status: Alpha

    This file is part of iPOPO.

    iPOPO is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    iPOPO is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with iPOPO. If not, see <http://www.gnu.org/licenses/>.
"""

# Documentation strings format
__docformat__ = "restructuredtext en"

# Module version
__version_info__ = (0, 5, 0)
__version__ = ".".join(map(str, __version_info__))

# ------------------------------------------------------------------------------

# Standard library
import logging
import threading
import sys

if sys.version_info[0] == 3:
    # Python 3
    import queue

else:
    # Python 2
    import Queue as queue

# ------------------------------------------------------------------------------

class FutureResult(object):
    """
    An object to wait for the result of a threaded execution
    """
    def __init__(self):
        """
        Sets up the FutureResult object
        """
        self._done_event = threading.Event()
        self._result = None


    def done(self):
        """
        Returns True if the job has finished, else False
        """
        return self._done_event.is_set()


    def result(self, timeout=None):
        """
        Waits up to timeout for the result the threaded job.
        Returns immediately the result if the job has already been done.
        
        :param timeout: The maximum time to wait for a result (in seconds)
        :raise OSError: The timeout raised before the job finished
        """
        if self._done_event.wait(timeout):
            return self._result

        raise OSError("Timeout raised")

# ------------------------------------------------------------------------------

class ThreadPool(object):
    """
    Executes the tasks stored in a FIFO in a thread pool
    """
    def __init__(self, nb_threads, queue_size=0, timeout=5, logname=None):
        """
        Sets up the task executor
        
        :param nb_threads: Size of the thread pool
        :param queue_size: Size of the task queue (0 for infinite)
        :param timeout: Queue timeout (in seconds)
        :param logname: Name of the logger
        :raise ValueError: Invalid number of threads
        """
        # Validate parameters
        if type(nb_threads) is not int or nb_threads < 1:
            raise ValueError("Invalid pool size: {0}".format(nb_threads))

        # The logger
        self._logger = logging.getLogger(logname or __name__)

        # The loop control event
        self._done_event = threading.Event()
        self._done_event.set()

        # The task queue
        self._queue = queue.Queue(queue_size)
        self._queue_size = queue_size
        self._timeout = timeout

        # The thread pool
        self._nb_threads = nb_threads
        self._threads = []


    def start(self):
        """
        Starts the thread pool. Does nothing if the pool is already started.
        """
        if not self._done_event.is_set():
            # Stop event not set: we're running
            return

        # Clear the stop event
        self._done_event.clear()

        # Create the threads
        i = 0
        while i < self._nb_threads:
            i += 1
            thread = threading.Thread(target=self.__run)
            self._threads.append(thread)

        # Start'em
        for thread in self._threads:
            thread.start()


    def stop(self):
        """
        Stops the thread pool. Does nothing if the pool is already stopped.
        """
        if self._done_event.is_set():
            # Stop event set: we're stopped
            return

        # Set the stop event
        self._done_event.set()

        # Add something in the queue (to unlock the join())
        try:
            for _ in self._threads:
                self._queue.put(self._done_event, True, self._timeout)

        except queue.Full:
            # There is already something in the queue
            pass

        # Join threads
        for thread in self._threads:
            thread.join()

        # Clear storage
        del self._threads[:]
        self.reset_queue()


    def enqueue(self, method, *args, **kwargs):
        """
        Enqueues a task in the pool
        
        :param method: Method to call
        :return: A FutureResult object, to get the result of the task
        :raise ValueError: Invalid method
        :raise Full: The task queue is full
        """
        if not hasattr(method, '__call__'):
            raise ValueError("{0} has no __call__ member." \
                             .format(method.__name__))

        # Prepare the future result object
        future = FutureResult()

        # Add the task to the queue
        self._queue.put((method, args, kwargs, future), True, self._timeout)
        return future


    def reset_queue(self):
        """
        Creates a new queue (deletes references to the old one)
        """
        self._queue = queue.Queue(self._queue_size)


    def __run(self):
        """
        The main loop
        """
        while not self._done_event.is_set():
            try:
                # Wait for an action (blocking)
                task = self._queue.get(True, self._timeout)
                if task is self._done_event:
                    # Stop event in the queue: get out
                    return

            except queue.Empty:
                # Nothing to do
                pass

            else:
                # Extract elements
                method, args, kwargs, future = task
                try:
                    # Call the method
                    future._result = method(*args, **kwargs)

                except Exception as ex:
                    logging.exception("Error executing %s: %s",
                                      method.__name__, ex)

                finally:
                    # Mark the action as executed
                    future._done_event.set()
                    self._queue.task_done()



#!/usr/bin/env python
# -- Content-Encoding: UTF-8 --
"""
Utility methods and decorators

:author: Thomas Calmant
:copyright: Copyright 2013, isandlaTech
:license: GPLv3
:version: 0.3.1
:status: Alpha

    This file is part of iPOPO.

    iPOPO is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    iPOPO is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with iPOPO. If not, see <http://www.gnu.org/licenses/>.
"""

from functools import wraps
from collections import deque

import logging
import sys
import threading
import traceback

# ------------------------------------------------------------------------------

# Module version
__version_info__ = (0, 3, 1)
__version__ = ".".join(map(str, __version_info__))

# Documentation strings format
__docformat__ = "restructuredtext en"

# Using Python 3
PYTHON_3 = (sys.version_info[0] == 3)

# ------------------------------------------------------------------------------

class Deprecated(object):
    """
    Prints a warning when using the decorated method
    """
    def __init__(self, message=None, logger=None):
        """
        Sets the deprecation message, e.g. to indicate which method to call
        instead.
        If a logger is given, its 'warning' method will be called to print the
        message; else the standard 'print' method will be used.
        
        :param message: Message to be printed
        :param logger: The name of the logger to use, or None.
        """
        self.__message = message or "Deprecated method"
        self.__logger = logger or None
        self.__already_logged = False


    def __log(self, method_name):
        """
        Logs the deprecation message on first call, does nothing after 
        
        :param method_name: Name of the deprecated method
        """
        if not self.__already_logged:
            # Print only if not already done
            stack = '\n\t'.join(traceback.format_stack())

            logging.getLogger(self.__logger).warning("%s: %s\n%s",
                                                     method_name,
                                                     self.__message,
                                                     stack)
            self.__already_logged = True


    def __call__(self, method):
        """
        Applies the modifications
        
        :param method: The decorated method
        :return: The wrapped method
        """
        # Prepare the wrapped call
        @wraps(method)
        def wrapped(*args, **kwargs):
            self.__log(method.__name__)
            return method(*args, **kwargs)

        return wrapped


# ------------------------------------------------------------------------------

class Synchronized(object):
    """
    A synchronizer for global methods
    """
    def __init__(self, lock=None):
        """
        Sets up the decorator. If 'lock' is None, an RLock() is created for
        this decorator.

        :param lock: The lock to be used for synchronization (can be None)
        """
        if not is_lock(lock):
            self.__lock = threading.RLock()

        else:
            self.__lock = lock


    def __call__(self, method):
        """
        Sets up the decorated method

        :param method: The decorated method
        :return: The wrapped method
        """

        @wraps(method)
        def wrapped(*args, **kwargs):
            """
            The wrapping method
            """
            with self.__lock:
                return method(*args, **kwargs)

        return wrapped


def SynchronizedClassMethod(*locks_attr_names, **kwargs):
    """
    A synchronizer decorator for class methods. An AttributeError can be raised
    at runtime if the given lock attribute doesn't exist or if it is None.
    
    If a parameter ``sorted`` is found in ``kwargs`` and its value is True,
    then the list of locks names will be sorted before locking.

    :param locks_attr_names: A list of the lock(s) attribute(s) name(s) to be
                             used for synchronization
    :return: The decorator method, surrounded with the lock
    """
    # Filter the names (remove empty ones)
    locks_attr_names = [lock_name
                        for lock_name in locks_attr_names
                        if lock_name]

    if not locks_attr_names:
        raise ValueError("The lock names list can't be empty")

    if 'sorted' not in kwargs or kwargs['sorted']:
        # Sort the lock names if requested
        # (locking always in the same order reduces the risk of dead lock)
        locks_attr_names = list(locks_attr_names)
        locks_attr_names.sort()

    def wrapped(method):
        """
        The wrapping method

        :param method: The wrapped method
        :return: The wrapped method
        :raise AttributeError: The given attribute name doesn't exist
        """
        @wraps(method)
        def synchronized(self, *args, **kwargs):
            """
            Calls the wrapped method with a lock
            """
            # Raises an AttributeError if needed
            locks = [getattr(self, attr_name) for attr_name in locks_attr_names]
            locked = deque()
            i = 0

            try:
                # Lock
                for lock in locks:
                    if lock is None:
                        # No lock...
                        raise AttributeError(
                            "Lock '{0}' can't be None in class {1}".format(
                                    locks_attr_names[i], type(self).__name__))

                    # Get the lock
                    i += 1
                    lock.acquire()
                    locked.appendleft(lock)

                # Use the method
                return method(self, *args, **kwargs)

            finally:
                # Unlock what has been locked in all cases
                for lock in locked:
                    lock.release()

                locked.clear()
                del locks[:]

        return synchronized

    # Return the wrapped method
    return wrapped

def is_lock(lock):
    """
    Tests if the given lock is an instance of a lock class
    """
    if lock is None:
        # Don't do useless tests
        return False

    for attr in ('acquire', 'release', '__enter__', '__exit__'):
        if not hasattr(lock, attr):
            # Missing something
            return False

    # Same API as a lock
    return True

# ------------------------------------------------------------------------------

def read_only_property(value):
    """
    Makes a read-only property that always returns the given value
    """
    return property(lambda cls: value)

# ------------------------------------------------------------------------------

def remove_all_occurrences(sequence, item):
    """
    Removes all occurrences of item in the given sequence

    :param sequence: The items list
    :param item: The item to be removed
    """
    if sequence is None:
        return

    while item in sequence:
        sequence.remove(item)

# ------------------------------------------------------------------------------

def add_listener(registry, listener):
    """
    Adds a listener in the registry, if it is not yet in

    :param registry: A registry (a list)
    :param listener: The listener to register
    :return: True if the listener has been added
    """
    if listener is None or listener in registry:
        return False

    registry.append(listener)
    return True


def remove_listener(registry, listener):
    """
    Removes a listener from the registry

    :param registry: A registry (a list)
    :param listener: The listener to remove
    :return: True if the listener was in the list
    """
    if listener is not None and listener in registry:
        registry.remove(listener)
        return True

    return False

# ------------------------------------------------------------------------------

if PYTHON_3:
    # Python 3 interpreter : bytes & str
    def is_string(string):
        """
        Utility method to test if the given parameter is a string
        (Python 2.x, 3.x) or a unicode (Python 2.x) object
    
        :param string: A potential string object
        :return: True if the given object is a string object or a Python 2.6
                 unicode object
        """
        # Python 3 only have the str string type
        return isinstance(string, str)

    def to_bytes(data, encoding="UTF-8"):
        """
        Converts the given string to an array of bytes.
        Returns the first parameter if it is already an array of bytes.
        
        :param data: A unicode string
        :param encoding: The encoding of data
        :return: The corresponding array of bytes
        """
        if type(data) is bytes:
            # Nothing to do
            return data

        return data.encode(encoding)


    def to_str(data, encoding="UTF-8"):
        """
        Converts the given parameter to a string.
        Returns the first parameter if it is already an instance of ``str``.
        
        :param data: A string
        :param encoding: The encoding of data
        :return: The corresponding string
        """
        if type(data) is str:
            # Nothing to do
            return data

        return str(data, encoding)

    # Same operation
    to_unicode = to_str

else:
    # Python 2 interpreter : str & unicode
    def is_string(string):
        """
        Utility method to test if the given parameter is a string
        (Python 2.x, 3.x) or a unicode (Python 2.x) object
    
        :param string: A potential string object
        :return: True if the given object is a string object or a Python 2.6
                 unicode object
        """
        # Python 2 also have unicode
        return isinstance(string, (str, unicode))


    def to_str(data, encoding="UTF-8"):
        """
        Converts the given parameter to a string.
        Returns the first parameter if it is already an instance of ``str``.
        
        :param data: A string
        :param encoding: The encoding of data
        :return: The corresponding string
        """
        if type(data) is str:
            # Nothing to do
            return data

        return data.encode(encoding)


    # Same operation
    to_bytes = to_str


    def to_unicode(data, encoding="UTF-8"):
        """
        Converts the given string to an unicode string using ``str.decode()``.
        Returns the first parameter if it is already an instance of ``unicode``.
        
        :param data: A string
        :param encoding: The encoding of data
        :return: The corresponding ``unicode`` string
        """
        if type(data) is unicode:
            # Nothing to do
            return data

        return data.decode(encoding)



#!/usr/bin/env python
# -- Content-Encoding: UTF-8 --
"""
Defines some iPOPO constants

:author: Thomas Calmant
:copyright: Copyright 2013, isandlaTech
:license: GPLv3
:version: 0.5
:status: Alpha

..

    This file is part of iPOPO.
    
    iPOPO is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.
    
    iPOPO is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
    
    You should have received a copy of the GNU General Public License
    along with iPOPO. If not, see <http://www.gnu.org/licenses/>.
"""

# Module version
__version_info__ = (0, 5, 0)
__version__ = ".".join(map(str, __version_info__))

# Documentation strings format
__docformat__ = "restructuredtext en"

# ------------------------------------------------------------------------------

IPOPO_SERVICE_SPECIFICATION = "pelix.ipopo.core"
""" iPOPO service specification string """

# ------------------------------------------------------------------------------

# Injected class fields
IPOPO_METHOD_CALLBACKS = "__ipopo_callbacks__"
""" Contains the list of callback types this method is decorated for """

IPOPO_METHOD_FIELD_CALLBACKS = "__ipopo_field_callbacks__"
""" Contains a list of tuples (field, callback type) """

IPOPO_FACTORY_CONTEXT = "__ipopo_factory_context__"
""" Storage of the FactoryContext object """

IPOPO_FACTORY_CONTEXT_DATA = "__ipopo_factory_context_data__"
""" Storage of the dictionary form of the FactoryContext """

IPOPO_INSTANCES = "__ipopo_instances__"
""" List of configured instantiations """

# Method called by the injected property (must be injected in the instance)
IPOPO_GETTER_SUFFIX = "_getter"
IPOPO_SETTER_SUFFIX = "_setter"
IPOPO_PROPERTY_PREFIX = "_ipopo_property"
IPOPO_CONTROLLER_PREFIX = "_ipopo_controller"

# ------------------------------------------------------------------------------

# Callbacks
IPOPO_CALLBACK_BIND = "BIND"
""" Bind: called when a dependency is injected """

IPOPO_CALLBACK_BIND_FIELD = "BIND_FIELD"
""" BindField: called when a dependency is injected in the given field """

IPOPO_CALLBACK_UPDATE = "UPDATE"
"""
Update: called when the properties of an injected dependency have been updated
"""

IPOPO_CALLBACK_UPDATE_FIELD = "UPDATE_FIELD"
"""
UpdateField: called when the properties of a dependency injected in the given
field have been updated
"""

IPOPO_CALLBACK_UNBIND = "UNBIND"
""" Unbind: called when a dependency is about to be removed """

IPOPO_CALLBACK_UNBIND_FIELD = "UNBIND_FIELD"
"""
UnbindFielld: called when a dependency is about to be removed from the given
field
"""

IPOPO_CALLBACK_VALIDATE = "VALIDATE"
""" Validate: Called when all mandatory dependencies have been bound """

IPOPO_CALLBACK_INVALIDATE = "INVALIDATE"
""" Invalidate: Called when one the mandatory dependencies is unbound """

# Properties
IPOPO_INSTANCE_NAME = "instance.name"
""" Name of the component instance """

IPOPO_REQUIRES_FILTERS = "requires.filters"
""" Dictionary to override @Requires filters """

IPOPO_AUTO_RESTART = "pelix.ipopo.auto_restart"
"""
If True, the component will be re-instantiated after its bundle has been updated
"""

# ------------------------------------------------------------------------------

HANDLER_DEPENDENCY = 'dependency'
"""
Represents the 'dependency' kind of handler.
Those handlers must implement the following methods:

* get_bindings(): Retrieves the list of bound service references
* is_valid(): Returns True if the dependency is in a valid state
"""

HANDLER_SERVICE_PROVIDER = 'service_provider'
"""
Represents the 'service_provider' kind of handler.
Those handlers must implement the following method:

* get_service_reference(): Retrieves the reference of the provided service
  (a ServiceReference object).

It should also implement the following ones:

* on_controller_changer(): Called when a component controller has been modified.
  The publication of a service might be stopped if its controller is set to
  False.
* on_property_change(): Called when a component property has been modified.
  The provided service properties should be modified accordingly.
"""

# ------------------------------------------------------------------------------

def get_ipopo_svc_ref(bundle_context):
    """
    Retrieves a tuple containing the service reference to iPOPO and the service
    itself
    
    :param bundle_context: The calling bundle context
    :return: The reference to the iPOPO service and the service itself,
             None if not available
    """
    # Late import
    import pelix.framework

    ref = bundle_context.get_service_reference(IPOPO_SERVICE_SPECIFICATION)
    if ref is None:
        return None

    try:
        svc = bundle_context.get_service(ref)

    except pelix.framework.BundleException:
        # Service reference has been invalidated
        return None

    return (ref, svc)

# ------------------------------------------------------------------------------

class IPopoEvent(object):
    """
    An iPOPO event descriptor.
    """
    REGISTERED = 1
    """ A component factory has been registered """

    INSTANTIATED = 2
    """ A component has been instantiated, but not yet validated """

    VALIDATED = 3
    """ A component has been validated """

    INVALIDATED = 4
    """ A component has been invalidated """

    BOUND = 5
    """ A reference has been injected in the component """

    UNBOUND = 6
    """ A reference has been removed from the component """

    KILLED = 9
    """ A component has been killed (removed from the list of instances) """

    UNREGISTERED = 10
    """ A component factory has been unregistered """


    def __init__(self, kind, factory_name, component_name):
        """
        Sets up the iPOPO event

        :param kind: Kind of event
        :param factory_name: Name of the factory associated to the event
        :param component_name: Name of the component instance associated to the
                               event
        """
        self.__kind = kind
        self.__factory_name = factory_name
        self.__component_name = component_name


    def get_component_name(self):
        """
        Retrieves the name of the component associated to the event

        :return: the name of the component
        """
        return self.__component_name


    def get_factory_name(self):
        """
        Retrieves the name of the factory associated to the event

        :return: the name of the component factory
        """
        return self.__factory_name


    def get_kind(self):
        """
        Retrieves the kind of event

        :return: the kind of event
        """
        return self.__kind



#!/usr/bin/env python
# -- Content-Encoding: UTF-8 --
"""
Pelix shell bundle.

Provides the basic command parsing and execution support to make a Pelix shell.

:author: Thomas Calmant
:copyright: Copyright 2013, isandlaTech
:license: GPLv3
:version: 0.2
:status: Alpha

..

    This file is part of iPOPO.

    iPOPO is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    iPOPO is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with iPOPO. If not, see <http://www.gnu.org/licenses/>.
"""

# Module version
__version_info__ = (0, 2, 1)
__version__ = ".".join(map(str, __version_info__))

# Documentation strings format
__docformat__ = "restructuredtext en"

# ------------------------------------------------------------------------------

# Shell constants
from pelix.shell import SHELL_SERVICE_SPEC, SHELL_COMMAND_SPEC, \
    SHELL_UTILS_SERVICE_SPEC

# Pelix modules
from pelix.utilities import to_str
import pelix.framework as pelix

# Standard library
import io
import logging
import os
import shlex
import sys
import traceback

# ------------------------------------------------------------------------------

DEFAULT_NAMESPACE = "default"
""" Default command name space: default """

_logger = logging.getLogger(__name__)

# ------------------------------------------------------------------------------

def _find_assignment(arg_token):
    """
    Find the first non-escaped assignment in the given argument token.
    Returns -1 if no assignment was found.

    :param arg_token: The argument token
    :return: The index of the first assignment, or -1
    """
    idx = arg_token.find('=')
    while idx != -1:
        if idx != 0:
            if arg_token[idx - 1] != '\\':
                # No escape character
                return idx

        idx = arg_token.find('=', idx + 1)

    # No assignment found
    return -1


def _make_args(args_list):
    """
    Converts the given list of arguments into a list (args) and a
    dictionary (kwargs).
    All arguments with an assignment are put into kwargs, others in args.

    :param args_list: The list of arguments to be treated
    :return: The (arg_token, kwargs) tuple.
    """
    args = []
    kwargs = {}

    for arg_token in args_list:
        idx = _find_assignment(arg_token)
        if idx != -1:
            # Assignment
            key = arg_token[:idx]
            value = arg_token[idx + 1:]
            kwargs[key] = value

        else:
            # Direct argument
            args.append(arg_token)

    return args, kwargs


def _split_ns_command(cmd_token):
    """
    Extracts the name space and the command name of the given command token.

    :param cmd_token: The command token
    :return: The extracted (name space, command) tuple
    """
    namespace = None
    cmd_split = cmd_token.split('.', 1)
    if len(cmd_split) == 1:
        # No name space given
        command = cmd_split[0]

    else:
        # Got a name space and a command
        namespace = cmd_split[0]
        command = cmd_split[1]

    if not namespace:
        # No name space given: given an empty one
        namespace = ""

    # Use lower case values only
    return namespace.lower(), command.lower()

# ------------------------------------------------------------------------------

class IOHandler(io.RawIOBase):
    """
    Handles I/O operations between the command handler and the client
    It automatically converts the given data to bytes in Python 3.
    """
    def __init__(self, instream, out_stream, encoding='UTF-8'):
        """
        Sets up the printer
        
        :param instream: Input stream
        :param out_stream: Output stream
        :param encoding: Output encoding
        """
        self.input = instream
        self.output = out_stream
        self.encoding = encoding

        # Set up class methods
        if sys.version_info[0] == 3:
            self.read = self._read_python3
            self.write = self._write_python3

        else:
            self.read = self.input.read
            self.write = self.output.write


    def _read_python3(self, n):
        """
        Reads the input stream
        
        :param n: Maximum bytes to read
        :return: The result of ``self.input.read()``
        """
        return to_str(self.input.read(n), self.encoding)


    def _write_python3(self, data):
        """
        Converts the given data then writes it
        
        :param data: Data to be written
        :return: The result of ``self.output.write()``
        """
        self.output.write(to_str(data, self.encoding))


    def write_line(self, line, *args, **kwargs):
        """
        Formats and writes a line to the output
        """
        if line is None:
            # Empty line
            line = '\n'

        else:
            # Format the line
            line = line.format(*args, **kwargs)

        # Add the trailing new line
        if line[-1] != '\n':
            line += '\n'

        # Write it
        self.write(line)


    def write_line_no_feed(self, line, *args, **kwargs):
        """
        Formats and writes a line to the output
        """
        if line is None:
            # Empty line
            line = ""

        else:
            # Format the line
            line = line.format(*args, **kwargs)

            # Remove the trailing line feed
            if line[-1] == '\n':
                line = line[:-1]

        # Write it
        self.write(line)

# ------------------------------------------------------------------------------

class ShellUtils(object):
    """
    Utility methods for the shell
    """
    def bundlestate_to_str(self, state):
        """
        Converts a bundle state integer to a string
        """
        states = {
                  pelix.Bundle.INSTALLED:   "INSTALLED",
                  pelix.Bundle.ACTIVE:      "ACTIVE",
                  pelix.Bundle.RESOLVED:    "RESOLVED",
                  pelix.Bundle.STARTING:    "STARTING",
                  pelix.Bundle.STOPPING:    "STOPPING",
                  pelix.Bundle.UNINSTALLED: "UNINSTALLED"
        }

        return states.get(state, "Unknown state (%d)".format(state))


    def make_table(self, headers, lines):
        """
        Generates an ASCII table according to the given headers and lines

        :param headers: List of table headers (N-tuple)
        :param lines: List of table lines (N-tuples).
        :return: The ASCII representation of the table
        :raise ValueError: Different number of columns between headers and lines
        """
        if lines and len(headers) != len(lines[0]):
            raise ValueError("Different sizes for header and lines")

        # Maximum lengths
        lengths = [len(title) for title in headers]

        # Lines
        str_lines = []
        for line in lines:
            # Recompute lengths
            i = 0
            str_line = []
            str_lines.append(str_line)
            for entry in line:
                str_entry = str(entry)
                str_line.append(str_entry)

                if len(str_entry) > lengths[i]:
                    lengths[i] = len(str_entry)
                i += 1

        # Prepare the head (centered text)
        format_str = "|"
        i = 0
        for length in lengths:
            format_str += " {%d:^%d} |" % (i, length)
            i += 1

        head_str = format_str.format(*headers)

        # Prepare the separator, according the length of the headers string
        separator = '-' * len(head_str)
        idx = head_str.find('|')
        while idx != -1:
            separator = '+'.join((separator[:idx], separator[idx + 1:]))
            idx = head_str.find('|', idx + 1)

        # Prepare the output
        output = []
        output.append(separator)
        output.append(head_str)
        output.append(separator.replace('-', '='))

        # Compute the lines
        format_str = format_str.replace('^', '<')
        for line in str_lines:
            output.append(format_str.format(*line))
            output.append(separator)

        # Force the last end of line
        output.append("")

        # Join'em
        return '\n'.join(output)

# ------------------------------------------------------------------------------

class Shell(object):
    """
    A simple shell, based on shlex.

    Allows to use name spaces.
    """
    def __init__(self, context, utilities):
        """
        Sets up the shell

        :param context: The bundle context
        """
        self._commands = {}
        self._context = context
        self._utils = utilities

        # Bound services: reference -> service
        self._bound_references = {}

        # Service reference -> (name space, [commands])
        self._reference_commands = {}

        # Register basic commands
        self.register_command(None, "bd", self.bundle_details)
        self.register_command(None, "bl", self.bundles_list)

        self.register_command(None, "sd", self.service_details)
        self.register_command(None, "sl", self.services_list)

        self.register_command(None, "start", self.start)
        self.register_command(None, "stop", self.stop)
        self.register_command(None, "update", self.update)
        self.register_command(None, "install", self.install)
        self.register_command(None, "uninstall", self.uninstall)

        self.register_command(None, "properties", self.properties_list)
        self.register_command(None, "property", self.property_value)

        self.register_command(None, "sysprops", self.environment_list)
        self.register_command(None, "sysprop", self.environment_value)

        self.register_command(None, "threads", self.threads_list)
        self.register_command(None, "thread", self.thread_details)

        self.register_command(None, "help", self.print_help)
        self.register_command(None, "?", self.print_help)

        self.register_command(None, "quit", self.quit)
        self.register_command(None, "close", self.quit)
        self.register_command(None, "exit", self.quit)


    def _bind_handler(self, svc_ref):
        """
        Called if a command service has been found.
        Registers the methods of this service.
        
        :param svc_ref: A reference to the found service
        :return: True if the commands have been registered
        """
        if svc_ref in self._bound_references:
            # Already bound service
            return False

        # Get the service
        handler = self._context.get_service(svc_ref)

        # Get its name space
        namespace = handler.get_namespace()
        commands = []

        # Get the host exporting the service, if any
        remote_host = None
        if svc_ref.get_property("service.imported"):
            # Imported service: store its host
            remote_host = svc_ref.get_property("service.imported.from")

        if not remote_host:
            # Local service: register all its methods directly
            for command, method in handler.get_methods():
                self.register_command(namespace, command, method)
                commands.append(command)

        else:
            # Imported service
            _logger.info("Bound to a remote command handler from %s",
                         remote_host)

            # Prefix its name space
            namespace = ".".join((remote_host, namespace))
            for command, method_name in handler.get_methods_names():
                # Use a proxy to call the methods
                def proxy(*args, **kwargs):
                    """
                    Remote command proxy
                    """
                    return getattr(handler, method_name)(*args, **kwargs)

                self.register_command(namespace, command, proxy)
                commands.append(command)

        # Store the reference
        self._bound_references[svc_ref] = handler
        self._reference_commands[svc_ref] = (namespace, commands)
        return True


    def _unbind_handler(self, svc_ref):
        """
        Called if a command service is gone.
        Unregisters its commands.
        
        :param svc_ref: A reference to the unbound service
        :return: True if the commands have been unregistered
        """
        if svc_ref not in self._bound_references:
            # Unknown reference
            return False

        # Unregister its commands
        namespace, commands = self._reference_commands[svc_ref]
        for command in commands:
            self.unregister(namespace, command)

        # Release the service
        self._context.unget_service(svc_ref)
        del self._bound_references[svc_ref]
        del self._reference_commands[svc_ref]
        return True


    def register_command(self, namespace, command, method):
        """
        Registers the given command to the shell.

        The namespace can be None, empty or "default"

        :param namespace: The command name space.
        :param command: The shell name of the command
        :param method: The method to call
        :return: True if the method has been registered, False if it was already
                 known or invalid
        """
        if not namespace:
            namespace = DEFAULT_NAMESPACE

        if not command:
            _logger.error("No command name given")
            return False

        if method is None:
            _logger.error("No method given for %s.%s", namespace, command)
            return False

        # Store everything in lower case
        namespace = namespace.lower()
        command = command.lower()

        if namespace not in self._commands:
            space = self._commands[namespace] = {}
        else:
            space = self._commands[namespace]

        if command in space:
            _logger.error("Command already registered: %s.%s", namespace,
                          command)
            return False

        space[command] = method
        return True


    def unregister(self, namespace, command=None):
        """
        Unregisters the given command. If command is None, the whole name space
        is unregistered.

        :param namespace: The command name space.
        :param command: The shell name of the command, or None
        :return: True if the command was known, else False
        """
        if not namespace:
            namespace = DEFAULT_NAMESPACE

        namespace = namespace.lower()

        if namespace not in self._commands:
            _logger.warning("Unknown name space: %s", namespace)
            return False

        if command is not None:
            # Remove the command
            if command not in self._commands[namespace]:
                _logger.warning("Unknown command: %s.%s", namespace, command)
                return False

            del self._commands[namespace][command]

            # Remove the name space if necessary
            if not self._commands[namespace]:
                del self._commands[namespace]

        else:
            # Remove the whole name space
            del self._commands[namespace]

        return True


    def __find_command_ns(self, command):
        """
        Returns the name spaces where the given command named is registered.
        If the command exists in the default name space, the returned list will
        only contain the default name space.
        Returns an empty list of the command is unknown
        
        :param command: A command name
        :return: A list of name spaces
        """
        if command in self._commands[DEFAULT_NAMESPACE]:
            # Command is in the default name space, use it directly
            return [DEFAULT_NAMESPACE]

        # Look for the spaces where the command name appears
        namespaces = []
        for namespace, commands in self._commands.items():
            if command in commands:
                namespaces.append(namespace)

        namespaces.sort()
        return namespaces


    def execute(self, cmdline, stdin=sys.stdin, stdout=sys.stdout):
        """
        Executes the command corresponding to the given line
        """
        # Split the command line
        if not cmdline:
            return False

        # Convert the line into a string
        cmdline = to_str(cmdline)

        line_split = shlex.split(cmdline, True, True)
        if not line_split:
            return False

        # Prepare the I/O handler
        io_handler = IOHandler(stdin, stdout)

        namespace, command = _split_ns_command(line_split[0])
        if not namespace:
            # Name space not given, look for the command
            spaces = self.__find_command_ns(command)
            if not spaces:
                # Unknown command
                _logger.warning("Unknown command: %s", command)
                io_handler.write_line("Unknown command: {0}", command)
                return False

            elif len(spaces) > 1:
                # Multiple possibilities
                io_handler.write_line("Multiple definitions for {0}: {1}",
                                      command, spaces)

            else:
                # Use the found name space
                namespace = spaces[0]

        # Get the content of the name space
        space = self._commands.get(namespace, None)
        if not space:
            _logger.warning("Unknown name space: %s", namespace)
            io_handler.write_line("Unknown name space {0}", namespace)
            return False

        # Get the method
        method = space.get(command, None)
        if method is None:
            _logger.warning("Unknown command: %s.%s", namespace, command)
            io_handler.write_line("Unknown command: {0}.{1}", namespace,
                                  command)
            return False

        # Make arguments and keyword arguments
        args, kwargs = _make_args(line_split[1:])

        # Execute it
        try:
            return method(io_handler, *args, **kwargs)

        except TypeError as ex:
            # Invalid arguments...
            _logger.exception("Error calling %s.%s: %s", namespace, command, ex)
            io_handler.write_line("Invalid method call: {0}", ex)
            return False

        except Exception as ex:
            # Error
            io_handler.write_line("{0}: {1}", type(ex).__name__, str(ex))
            return False


    def get_banner(self):
        """
        Returns the Shell banner
        """
        return "** Pelix Shell prompt **"


    def get_ps1(self):
        """
        Returns the PS1, the basic shell prompt
        """
        return "$ "


    def get_namespaces(self):
        """
        Retrieves the list of known name spaces (without the default one)
        
        :return: The list of known name spaces
        """
        namespaces = list(self._commands.keys())
        namespaces.remove(DEFAULT_NAMESPACE)
        namespaces.sort()
        return namespaces


    def get_commands(self, namespace):
        """
        Retrieves the commands of the given name space. If *namespace* is None
        or empty, it retrieves the commands of the default name space
        
        :param namespace: The commands name space
        :return: A list of commands names
        """
        if not namespace:
            # Default name space:
            namespace = DEFAULT_NAMESPACE

        commands = list(self._commands[namespace].keys())
        commands.sort()
        return commands


    def bundle_details(self, io_handler, bundle_id):
        """
        Prints the details of the given bundle
        """
        try:
            bundle_id = int(bundle_id)
            bundle = self._context.get_bundle(bundle_id)

        except pelix.BundleException:
            io_handler.write_line("Unknown bundle ID: {0}", bundle_id)
            return

        lines = []
        lines.append("ID      : {0}".format(bundle.get_bundle_id()))
        lines.append("Name    : {0}".format(bundle.get_symbolic_name()))
        lines.append("Version : {0}".format(bundle.get_version()))
        lines.append("State   : {0}".format(self._utils.bundlestate_to_str(
                                                        bundle.get_state())))
        lines.append("Location: {0}".format(bundle.get_location()))
        lines.append("Published services:")
        try:
            services = bundle.get_registered_services()
            if services:
                for svc_ref in services:
                    lines.append("\t{0}".format(svc_ref))

            else:
                lines.append("\tn/a")

        except pelix.BundleException as ex:
            # Bundle in a invalid state
            lines.append("\tError: {0}".format(ex))

        lines.append("Services used by this bundle:")
        try:
            services = bundle.get_services_in_use()
            if services:
                for svc_ref in services:
                    lines.append("\t{0}".format(svc_ref))

            else:
                lines.append("\tn/a")

        except pelix.BundleException as ex:
            # Bundle in a invalid state
            lines.append("\tError: {0}".format(ex))

        lines.append("")
        io_handler.write('\n'.join(lines))


    def bundles_list(self, io_handler):
        """
        Lists the bundles in the framework and their state
        """
        # Head of the table
        headers = ('ID', 'Name', 'State', 'Version')

        # Get the bundles
        bundles = self._context.get_bundles()

        # The framework is not in the result of get_bundles()
        bundles.insert(0, self._context.get_bundle(0))

        # Make the entries
        lines = []
        for bundle in bundles:
            # Make the line
            line = [str(entry) for entry in
                    (bundle.get_bundle_id(), bundle.get_symbolic_name(),
                    self._utils.bundlestate_to_str(bundle.get_state()),
                    bundle.get_version())]

            lines.append(line)

        # Print'em all
        io_handler.write(self._utils.make_table(headers, lines))


    def service_details(self, io_handler, service_id):
        """
        Prints the details of the given service
        """
        svc_ref = self._context.get_service_reference(None,
                                '({0}={1})'.format(pelix.SERVICE_ID,
                                                   service_id))
        if svc_ref is None:
            io_handler.write_line('Service not found: {0}', service_id)
            return

        lines = []
        lines.append("ID    : {0}".format(svc_ref.get_property(
                                            pelix.SERVICE_ID)))
        lines.append("Rank  : {0}".format(svc_ref.get_property(
                                            pelix.SERVICE_RANKING)))
        lines.append("Specs : {0}".format(svc_ref.get_property(
                                            pelix.OBJECTCLASS)))
        lines.append("Bundle: {0}".format(svc_ref.get_bundle()))
        lines.append("Properties:")
        for key, value in svc_ref.get_properties().items():
            lines.append("\t{0} = {1}".format(key, value))

        lines.append("Bundles using this service:")
        for bundle in svc_ref.get_using_bundles():
            lines.append("\t{0}".format(bundle))

        lines.append("")
        io_handler.write('\n'.join(lines))


    def services_list(self, io_handler):
        """
        Lists the services in the framework
        """
        # Head of the table
        headers = ('ID', 'Specifications', 'Bundle', 'Ranking')

        # Lines
        references = self._context.get_all_service_references(None, None)

        # Use the reverse order (ascending service IDs instead of descending)
        references.reverse()

        lines = []
        for ref in references:
            # Make the line
            line = [str(entry) for entry in
                    (ref.get_property(pelix.SERVICE_ID),
                     ref.get_property(pelix.OBJECTCLASS),
                     ref.get_bundle(),
                     ref.get_property(pelix.SERVICE_RANKING))]

            lines.append(line)

        # Print'em all
        io_handler.write(self._utils.make_table(headers, lines))


    def print_help(self, io_handler):
        """
        Prints the available methods and their documentation
        """
        namespaces = list(self._commands.keys())
        namespaces.remove(DEFAULT_NAMESPACE)
        namespaces.sort()
        namespaces.insert(0, DEFAULT_NAMESPACE)

        for namespace in namespaces:
            io_handler.write_line("* Namespace '{0}':", namespace)
            names = [command for command in self._commands[namespace]]
            names.sort()

            for name in names:
                io_handler.write_line("- {0}", name)
                doc = getattr(self._commands[namespace][name], '__doc__', \
                              None) or "(Documentation missing)"
                io_handler.write_line("\t\t{0}", ' '.join(doc.split()))


    def properties_list(self, io_handler):
        """
        Lists the properties of the framework
        """
        # Get the framework
        framework = self._context.get_bundle(0)

        # Head of the table
        headers = ('Property Name', 'Value')

        # Lines
        lines = [item for item in framework.get_properties().items()]

        # Sort lines
        lines.sort()

        # Print the table
        io_handler.write(self._utils.make_table(headers, lines))


    def property_value(self, io_handler, name):
        """
        property <name> - Prints the value of the given property, looking into
        framework properties then environment variables.
        """
        io_handler.write_line(self._context.get_property(name))


    def environment_list(self, io_handler):
        """
        Lists the framework process environment variables
        """
        # Head of the table
        headers = ('Environment Variable', 'Value')

        # Lines
        lines = [item for item in os.environ.items()]

        # Sort lines
        lines.sort()

        # Print the table
        io_handler.write(self._utils.make_table(headers, lines))


    def environment_value(self, io_handler, name):
        """
        sysprop <name> - Prints the value of the given environment variable
        """
        io_handler.write_line(os.getenv(name))


    def threads_list(self, io_handler):
        """
        Lists the active threads and their current code line
        """
        # Extract frames
        frames = sys._current_frames()

        # Sort by thread ID
        thread_ids = list(frames.keys())
        thread_ids.sort()

        lines = []
        for thread_id in thread_ids:
            # Get the corresponding stack
            stack = frames[thread_id]

            # Construct the code position
            lines.append('Thread ID: {0}'.format(thread_id))
            lines.append('Stack trace:')
            lines.extend((line.rstrip()
                          for line in traceback.format_stack(stack, 8)))
            lines.append('')

        lines.append('')

        # Sort the lines
        io_handler.write('\n'.join(lines))


    def thread_details(self, io_handler, thread_id):
        """
        thread <id> - Prints details about the given thread
        """
        try:
            stack = sys._current_frames()[int(thread_id)]

        except KeyError:
            io_handler.write_line("Unknown thread ID: {0}", thread_id)

        except ValueError:
            io_handler.write_line("Invalid thread ID: {0}", thread_id)

        else:
            lines = []
            lines.append('Thread ID: {0}'.format(thread_id))
            lines.append('Stack trace:')
            lines.extend((line.rstrip()
                          for line in traceback.format_stack(stack)))

        lines.append("")
        io_handler.write('\n'.join(lines))


    def quit(self, io_handler):
        """
        Stops the current shell session (raises a KeyboardInterrupt exception)
        """
        raise KeyboardInterrupt()


    def start(self, io_handler, bundle_id):
        """
        start <bundle_id> - Starts the given bundle ID
        """
        bundle_id = int(bundle_id)
        bundle = self._context.get_bundle(bundle_id)
        if bundle is None:
            io_handler.write_line("Unknown bundle: {0}", bundle_id)

        bundle.start()


    def stop(self, io_handler, bundle_id):
        """
        stop <bundle_id> - Stops the given bundle
        """
        bundle_id = int(bundle_id)
        bundle = self._context.get_bundle(bundle_id)
        if bundle is None:
            io_handler.write_line("Unknown bundle: {0}", bundle_id)

        bundle.stop()


    def update(self, io_handler, bundle_id):
        """
        update <bundle_id> - Updates the given bundle ID
        """
        bundle_id = int(bundle_id)
        bundle = self._context.get_bundle(bundle_id)
        if bundle is None:
            io_handler.write_line("Unknown bundle: {0}", bundle_id)

        bundle.update()


    def install(self, io_handler, location):
        """
        install <bundle_id> - Installs the given bundle
        """
        bundle = self._context.install_bundle(location)
        io_handler.write_line("Bundle ID: {0}", bundle.get_bundle_id())


    def uninstall(self, io_handler, bundle_id):
        """
        uninstall <bundle_id> - Uninstalls the given bundle
        """
        bundle_id = int(bundle_id)
        bundle = self._context.get_bundle(bundle_id)
        if bundle is None:
            io_handler.write_line("Unknown bundle: {0}", bundle_id)

        bundle.uninstall()


# ------------------------------------------------------------------------------

class PelixActivator(object):
    """
    Activator class for Pelix
    """
    def __init__(self):
        """
        Sets up the activator
        """
        self._shell = None
        self._shell_reg = None
        self._utils_reg = None


    def service_changed(self, event):
        """
        Called when a command provider service event occurred
        """
        kind = event.get_kind()
        reference = event.get_service_reference()

        if kind in (pelix.ServiceEvent.REGISTERED,
                    pelix.ServiceEvent.MODIFIED):
            # New or modified service
            self._shell._bind_handler(reference)

        else:
            # Service gone or not matching anymore
            self._shell._unbind_handler(reference)


    def start(self, context):
        """
        Bundle starting

        :param context: The bundle context
        """
        try:
            # Prepare the shell utility service
            utils = ShellUtils()
            self._shell = Shell(context, utils)

            self._shell_reg = context.register_service(SHELL_SERVICE_SPEC,
                                                       self._shell, {})

            self._utils_reg = context.register_service(SHELL_UTILS_SERVICE_SPEC,
                                                       utils, {})

            # Register the service listener
            spec_filter = '({0}={1})'.format(pelix.OBJECTCLASS,
                                             SHELL_COMMAND_SPEC)
            context.add_service_listener(self, spec_filter)

            # Register existing command services
            refs = context.get_all_service_references(SHELL_COMMAND_SPEC,
                                                      None)
            if refs is not None:
                for ref in refs:
                    self._shell._bind_handler(ref)

            _logger.info("Shell services registered")

        except pelix.BundleException as ex:
            _logger.exception("Error registering the shell service: %s", ex)


    def stop(self, context):
        """
        Bundle stopping

        :param context: The bundle context
        """
        # Unregister the service listener
        context.remove_service_listener(self)

        # Unregister the services
        if self._shell_reg is not None:
            self._shell_reg.unregister()
            self._shell_reg = None

        if self._utils_reg is not None:
            self._utils_reg.unregister()
            self._utils_reg = None

        self._shell = None
        _logger.info("Shell services unregistered")


# Activator for Pelix
activator = PelixActivator()



#!/usr/bin/env python
# -- Content-Encoding: UTF-8 --
"""
Pelix basic HTTP service bundle.

Provides an implementation of the Pelix HTTP service based on the standard
Python library.

:author: Thomas Calmant
:copyright: Copyright 2013, isandlaTech
:license: GPLv3
:version: 0.1.1
:status: Alpha

..

    This file is part of iPOPO.

    iPOPO is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    iPOPO is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with iPOPO. If not, see <http://www.gnu.org/licenses/>.
"""

# Module version
__version_info__ = (0, 1, 1)
__version__ = ".".join(map(str, __version_info__))

# Documentation strings format
__docformat__ = "restructuredtext en"

# ------------------------------------------------------------------------------

from pelix.ipopo.decorators import ComponentFactory, Provides, Validate, \
    Invalidate, Property, Requires, BindField, UnbindField, UpdateField

import pelix.ipopo.constants as constants
import pelix.utilities as utilities

import logging
import socket
import sys
import threading
import traceback

# Basic HTTP server
if sys.version_info[0] == 3:
    # Python 3
    import urllib.parse as urlparse
    from http.server import HTTPServer
    from http.server import BaseHTTPRequestHandler
    from socketserver import ThreadingMixIn

else:
    # Python 2
    import urlparse
    from BaseHTTPServer import HTTPServer
    from BaseHTTPServer import BaseHTTPRequestHandler
    from SocketServer import ThreadingMixIn

# ------------------------------------------------------------------------------

# HTTP service constants
import pelix.http as http

HTTP_SERVICE_COMPONENT_FACTORY = "pelix.http.service.basic.factory"
""" Name of the HTTP service component factory """

# ------------------------------------------------------------------------------

class _HTTPServletRequest(http.AbstractHTTPServletRequest):
    """
    HTTP Servlet request helper
    """
    def __init__(self, request_handler):
        """
        Sets up the request helper
        
        :param request_handler: The basic request handler
        """
        self._handler = request_handler


    def get_client_address(self):
        """
        Retrieves the address of the client
        
        :return: A (host, port) tuple
        """
        return self._handler.client_address


    def get_header(self, name, default=None):
        """
        Retrieves the value of a header
        """
        return self._handler.headers.get(name, default)


    def get_headers(self):
        """
        Retrieves all headers
        """
        return self._handler.headers


    def get_path(self):
        """
        Retrieves the request full path
        """
        return self._handler.path


    def get_rfile(self):
        """
        Retrieves the input as a file stream
        """
        return self._handler.rfile


class _HTTPServletResponse(http.AbstractHTTPServletResponse):
    """
    HTTP Servlet response helper
    """
    def __init__(self, request_handler):
        """
        Sets up the response helper
        
        :param request_handler: The basic request handler
        """
        self._handler = request_handler


    def set_response(self, code, message=None):
        """
        Sets the response line.
        This method should be the first called when sending an answer.
        
        :param code: HTTP result code
        :param message: Associated message
        """
        self._handler.send_response(code, message)


    def set_header(self, name, value):
        """
        Sets the value of a header.
        This method should not be called after ``end_headers()``.
        
        :param name: Header name
        :param value: Header value
        """
        self._handler.send_header(name, value)


    def end_headers(self):
        """
        Ends the headers part
        """
        self._handler.end_headers()


    def get_wfile(self):
        """
        Retrieves the output as a file stream.
        ``end_headers()`` should have been called before, except if you want
        to write your own headers.
        
        :return: The output file-like object
        """
        return self._handler.wfile


    def write(self, data):
        """
        Writes the given data.
        ``end_headers()`` should have been called before, except if you want
        to write your own headers.
        
        :param data: Data to be written
        """
        self._handler.wfile.write(data)

# ------------------------------------------------------------------------------

class _RequestHandler(BaseHTTPRequestHandler):
    """
    Basic HTTP server request handler
    """
    # Override the default HTTP version
    default_request_version = "HTTP/1.0"

    def __init__(self, http_svc, *args, **kwargs):
        """
        Sets up the request handler (called for each request)
        
        :param http_svc: The associated HTTP service
        """
        self._service = http_svc

        # This calls the do_* methods
        BaseHTTPRequestHandler.__init__(self, *args, **kwargs)


    def __getattr__(self, name):
        """
        Retrieves the do_* in the servlet corresponding to the request path.
        If the name is not a "do_*", returns the normal result of __getattr__.
        
        :param name: Name of the attribute
        :return: The found attribute
        :raise AttributeError: Attribute not found
        """
        if not name.startswith("do_"):
            # Not a request handling
            return object.__getattribute__(self, name)

        # Parse the URL
        parsed_url = urlparse.urlparse(self.path)
        parsed_path = parsed_url.path

        # Get the corresponding servlet
        servlet_info = self._service.get_servlet(parsed_path)
        if servlet_info is not None:
            servlet = servlet_info[0]
            if hasattr(servlet, name):
                # Prepare the helpers
                request = _HTTPServletRequest(self)
                response = _HTTPServletResponse(self)

                # Create a wrapper to pass the handler to the servlet
                def wrapper():
                    try:
                        # Handle the request
                        return getattr(servlet, name)(request, response)

                    except:
                        # Send a 500 error page on error
                        return self.send_exception(response)

                # Return it
                return wrapper

        # Return the super implementation if needed
        return self.send_no_servlet_response


    def log_error(self, message, *args, **kwargs):
        """
        Log server error
        """
        self._service.log(logging.ERROR, message, *args, **kwargs)


    def log_request(self, code='-', size='-'):
        """
        Logs a request to the server
        """
        self._service.log(logging.DEBUG, '"%s" %s', self.requestline, code)


    def send_no_servlet_response(self):
        """
        Default response sent when no servlet is found for the requested path
        """
        page = """<html>
<head>
<title>404 - Page not found</title>
</head>
<body>
<h1>Page not found</h1>
<p>No servlet is associated to this path: <pre>{0}</pre></p>
</body>
</html>""".format(self.path)

        # Use the helper to send the error page
        response = _HTTPServletResponse(self)
        response.send_content(404, page)


    def send_exception(self, response):
        """
        Sends an exception page with a 500 error code.
        Must be called from inside the exception handling block.
        
        :param response: The response handler
        """
        # Get a formatted stack trace
        stack = traceback.format_exc()

        # Prepare the page content
        page = """<html>
<head>
<title>500 - Internal Server Error</title>
</head>
<body>
<h1>Internal Server Error</h1>
<p>Error hanling request upon: {0}</p>
<pre>
{1}
</pre>
</body>
</html>""".format(self.path, stack)

        # Send the page
        response.send_content(500, page)

# ------------------------------------------------------------------------------

class _HttpServerFamily(ThreadingMixIn, HTTPServer):
    """
    A small modification to have a threaded HTTP Server with a custom address
    family
    
    Inspired from:
    http://www.arcfn.com/2011/02/ipv6-web-serving-with-arc-or-python.html
    """
    def __init__(self, server_address, request_handler_class, logger=None):
        """
        Proxy constructor
        
        :param server_address: The server address
        :param request_handler_class: The request handler class
        :param logger: An optional logger, in case of ignored error
        """
        # Determine the address family
        addr_info = socket.getaddrinfo(server_address[0], server_address[1],
                                       0, 0, socket.SOL_TCP)

        # Change the address family before the socket is created
        # Get the family of the first possibility
        self.address_family = addr_info[0][0]

        # Special case: IPv6
        ipv6 = (self.address_family == socket.AF_INET6)

        # Set up the server, socket, ... but do not bind immediately
        HTTPServer.__init__(self, server_address, request_handler_class, False)

        if ipv6:
            # Explicitly ask to be accessible both by IPv4 and IPv6
            # Some versions of Python don't have V6ONLY.
            # On Linux, IPC6_V6ONLY = 26
            IPV6_V6ONLY = getattr(socket, "IPV6_V6ONLY", 26)

            try:
                self.socket.setsockopt(socket.IPPROTO_IPV6, IPV6_V6ONLY, 0)

            except socket.error as ex:
                # Ignore the error, but log it if possible
                if logger is not None:
                    logger.exception("Couldn't set IP double stack flag: %s",
                                     ex)

        # Bind & accept
        self.server_bind()
        self.server_activate()

# ------------------------------------------------------------------------------

@ComponentFactory(name=HTTP_SERVICE_COMPONENT_FACTORY)
@Provides(specifications=http.HTTP_SERVICE)
@Requires("_servlets_services", http.HTTP_SERVLET, True, True)
@Property("_address", http.HTTP_SERVICE_ADDRESS, "0.0.0.0")
@Property("_port", http.HTTP_SERVICE_PORT, 8080)
@Property("_instance_name", constants.IPOPO_INSTANCE_NAME)
@Property("_logger_name", "pelix.http.logger.name", "")
@Property("_logger_level", "pelix.http.logger.level", None)
class HttpService(object):
    """
    Basic HTTP service component
    """
    def __init__(self):
        """
        Constructor
        """
        # Properties
        self._address = "0.0.0.0"
        self._port = 8080
        self._instance_name = None
        self._logger_name = None
        self._logger_level = None

        # Validation flag
        self._validated = False

        # The logger
        self._logger = None

        # Servlets registry lock
        self._lock = threading.Lock()

        # Path -> (servlet, parameters)
        self._servlets = {}

        # Field injected by iPOPO
        self._servlets_services = None

        # Servlet -> ServiceReference
        self._servlets_refs = {}
        self._binding_lock = threading.Lock()

        # Server control
        self._server = None
        self._thread = None


    def __str__(self):
        """
        String representation of the instance
        """
        return "BasicHttpService({0}, {1:d})".format(self._address, self._port)


    def __safe_callback(self, instance, method, *args, **kwargs):
        """
        Safely calls the given method in the given instance.
        Returns True on method absence.
        Returns False on error.
        Returns the method result if found.
        
        :param instance: The instance to call
        :param method: The method to call in the instance
        :return: The method result or True on method absence or False on error
        """
        # Call back the method
        if instance is None:
            # Consider invalidity as a failure
            return False

        callback = getattr(instance, method, None)
        if callback is None:
            # Consider absence as a success
            return True

        try:
            result = callback(*args, **kwargs)
            if result is None:
                # Special case: consider None as success
                return True

            return result

        except Exception as ex:
                self.log_exception("Error calling back an instance: %s", ex)

        return False


    def __register_servlet_service(self, service, service_reference):
        """
        Registers a servlet according to its service properties
        
        :param service: A servlet service
        :param service_reference: The associated ServiceReference
        """
        # Servlet bound
        paths = service_reference.get_property(http.HTTP_SERVLET_PATH)
        if utilities.is_string(paths):
            # Register the servlet to a single path
            self.register_servlet(paths, service, None)

        elif isinstance(paths, (list, tuple)):
            # Register the servlet to multiple paths
            for path in paths:
                self.register_servlet(path, service, None)


    @BindField("_servlets_services")
    def _bind(self, field, service, service_reference):
        """
        Called by iPOPO when a service is bound
        """
        with self._binding_lock:
            self._servlets_refs[service] = service_reference

            if self._validated:
                # We've been validated, register the service
                self.__register_servlet_service(service, service_reference)


    @UpdateField('_servlets_services')
    def _update(self, field, service, service_reference, old_properties):
        """
        Called by iPOPO when the properties of a service have been updated
        """
        # Check if the property concerns the registration
        old_path = old_properties.get(http.HTTP_SERVLET_PATH)
        new_path = service_reference.get_property(http.HTTP_SERVLET_PATH)
        if old_path == new_path:
            # Nothing to do
            return

        with self._binding_lock:
            # Unregister the previous paths
            self.unregister(None, service)

            if self._validated:
                # Register the service with its new properties
                self.__register_servlet_service(service, service_reference)


    @UnbindField("_servlets_services")
    def _unbind(self, field, service, service_reference):
        """
        Called by iPOPO when a service is gone
        """
        with self._binding_lock:
            # Servlet gone: unregister all paths associated to this servlet
            self.unregister(None, service)

            # Remove the service reference
            del self._servlets_refs[service]


    def get_access(self):
        """
        Retrieves the (address, port) tuple to access the server
        """
        sock_info = self._server.socket.getsockname()

        # Only keep the address and the port information
        return sock_info[0], sock_info[1]


    def get_hostname(self):
        """
        Retrieves the server host name
        
        :return: The server host name
        """
        return socket.gethostname()


    def get_servlet(self, path):
        """
        Retrieves the servlet matching the given path and its parameters.
        Returns None if no servlet matches the given path.
        
        :param path: A request URI
        :return: A tuple (servlet, parameters) or None
        """
        if not path or path[0] != "/":
            # No path, nothing to return
            return None

        # Use lower case for comparison
        path = path.lower()

        if path[-1] != '/':
            # Add a trailing slash
            path += '/'

        with self._lock:
            longest_match = ""
            for servlet_path in self._servlets.keys():

                tested_path = servlet_path
                if tested_path[-1] != '/':
                    # Add a trailing slash
                    tested_path += '/'

                if path.startswith(tested_path):
                    # Found a corresponding servlet
                    if len(servlet_path) > len(longest_match):
                        # And its deeper than the previous one
                        longest_match = servlet_path

            # Return the found servlet
            if not longest_match:
                # No match found
                return None

            else:
                # Retrieve the stored information
                return self._servlets[longest_match]


    def register_servlet(self, path, servlet, parameters=None):
        """
        Registers a servlet
        
        :param path: Path handled by this servlet
        :param servlet: The servlet instance
        :param parameters: The parameters associated to this path
        :return: True if the servlet has been registered, False if it refused
                 the binding.
        :raise ValueError: Invalid path or handler
        """
        if servlet is None:
            raise ValueError("Invalid servlet instance")

        if not path or path[0] != '/':
            raise ValueError("Invalid path given to register the servlet: {0}" \
                             .format(path))

        # Use lower-case paths
        path = path.lower()

        # Prepare the parameters
        if parameters is None:
            parameters = {}

        with self._lock:
            if path in self._servlets:
                # Already registered path
                if self._servlets[path][0] is servlet:
                    # Nothing to do
                    return True

                else:
                    raise ValueError("A servlet is already registered on {0}" \
                                     .format(path))

            # Add server information in parameters
            parameters['http.address'] = self._address
            parameters['http.port'] = self._port

            # Call back the method
            if self.__safe_callback(servlet, "bound_to", path, parameters):
                # Store the servlet
                self._servlets[path] = (servlet, parameters)
                return True

            return False


    def unregister(self, path, servlet=None):
        """
        Unregisters the servlet for the given path
        
        :param path: The path to a servlet
        :param servlet: If given, unregisters all the paths handled by this
                        servlet
        
        :return: True if at least one path as been unregistered, else False
        """
        if servlet is not None:
            with self._lock:
                # Unregister all paths for this servlet
                paths = [path
                         for (path, servlet_info) in self._servlets.items()
                         if servlet_info[0] == servlet]

            result = False
            for path in paths:
                result |= self.unregister(path)

            return result

        else:
            if not path:
                # Invalid path
                return False

            # Always use lower case to compare paths
            path = path.lower()

            with self._lock:
                # Notify the servlet
                servlet_info = self._servlets.get(path, None)
                if servlet_info is None:
                    # Unknown path
                    return False

                self.__safe_callback(servlet_info[0], "unbound_from",
                                     path, servlet_info[1])

                # Remove the servlet
                del self._servlets[path]
                return True


    def log(self, level, message, *args, **kwargs):
        """
        Logs the given message
        
        :param level: Log entry level
        :param message: Log message (Python logging format)
        """
        if self._logger is not None:
            # Log the message
            self._logger.log(level, message, *args, **kwargs)


    def log_exception(self, message, *args, **kwargs):
        """
        Logs an exception
        
        :param message: Log message (Python logging format)
        """
        if self._logger is not None:
            # Log the exception
            self._logger.exception(message, *args, **kwargs)


    @Validate
    def validate(self, context):
        """
        Component validation
        """
        if not self._address:
            # Local host by default
            self._address = "localhost"

        if self._port is None or self._port < 0:
            # Random port
            self._port = 0

        else:
            # Ensure we have an integer
            self._port = int(self._port)

        # Set up the logger
        if self._logger_name is not None:
            if not self._logger:
                # Empty name, use the instance name
                self._logger_name = self._instance_name

            self._logger = logging.getLogger(self._logger_name)

            if self._logger_level is None:
                self._logger.level = logging.INFO

            else:
                self._logger.level = int(self._logger_level)

        self.log(logging.INFO, "Starting HTTP server: [%s]:%d ...",
                 self._address, self._port)

        # Create the server
        self._server = _HttpServerFamily((self._address, self._port),
                                         lambda *x: _RequestHandler(self, *x),
                                         self._logger)

        # Property update (if port was 0)
        self._port = self._server.socket.getsockname()[1]

        # Run it in a separate thread
        self._thread = threading.Thread(target=self._server.serve_forever)
        self._thread.daemon = True
        self._thread.start()

        with self._binding_lock:
            # Set the validation flag up, once the server is ready
            self._validated = True

            # Register bound servlets
            for service, svc_ref in self._servlets_refs.items():
                self.__register_servlet_service(service, svc_ref)

        self.log(logging.INFO, "HTTP server started: [%s]:%d ...",
                 self._address, self._port)


    @Invalidate
    def invalidate(self, context):
        """
        Component invalidation
        """
        with self._binding_lock:
            # Refuse new registrations
            self._validated = False

            # Unregister servlets (to call unbound_from...)
            for service in self._servlets_refs:
                self.unregister(None, service)

        self.log(logging.INFO, "Shutting down HTTP server: [%s]:%d ...",
                 self._address, self._port)

        # Shutdown server
        self._server.shutdown()

        # Wait for the thread to stop...
        self.log(logging.INFO,
                 "Waiting HTTP server ([%s]:%d) thread to stop...",
                 self._address, self._port)
        self._thread.join(2)

        # Close the server
        self._server.server_close()

        self.log(logging.INFO, "HTTP server down: [%s]:%d ...",
                 self._address, self._port)

        # Clean up
        self._servlets.clear()
        self._thread = None
        self._server = None
        self._logger = None



#!/usr/bin/env python
# -- Content-Encoding: UTF-8 --
"""
Defines the iPOPO decorators classes to manipulate component factory classes

:author: Thomas Calmant
:copyright: Copyright 2013, isandlaTech
:license: GPLv3
:version: 0.5
:status: Alpha

..

    This file is part of iPOPO.

    iPOPO is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.
    
    iPOPO is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
    
    You should have received a copy of the GNU General Public License
    along with iPOPO. If not, see <http://www.gnu.org/licenses/>.
"""

# Module version
__version_info__ = (0, 5, 0)
__version__ = ".".join(map(str, __version_info__))

# Documentation strings format
__docformat__ = "restructuredtext en"

# ------------------------------------------------------------------------------

# Pelix modules
from pelix.utilities import is_string
from pelix.ipopo.core import FactoryContext, Requirement
import pelix.ipopo.constants as constants

# Standard library
import inspect
import logging
import threading
import types

# ------------------------------------------------------------------------------

# Prepare the module logger
_logger = logging.getLogger("ipopo.decorators")

# ------------------------------------------------------------------------------

def is_from_parent(cls, attribute_name):
    """
    Tests if the current attribute value is shared by a parent of the given
    class.
    
    Returns None if the attribute value is None.
    
    :param cls: Child class with the requested attribute
    :param attribute_name: Name of the attribute to be tested
    :return: True if the attribute value is shared with a parent class
    """
    value = getattr(cls, attribute_name, None)
    if value is None:
        # No need to go further
        return False

    for base in cls.__bases__:
        base_value = getattr(base, attribute_name, None)
        if base_value is value:
            # Found !
            return True

    # Attribute value not found in parent classes
    return False


def get_method_description(method):
    """
    Retrieves a description of the given method. If possible, the description
    contains the source file name and line.
    
    :param method: A method
    :return: A description of the method (at least its name)
    """
    try:
        try:
            line_no = inspect.getsourcelines(method)[1]

        except IOError:
            # Error reading the source file
            line_no = -1

        return "'{method}' ({file}:{line})".format(method=method.__name__,
                                                   file=inspect.getfile(method),
                                                   line=line_no)

    except TypeError:
        # Method can't be inspected
        return "'{0}'".format(method.__name__)


def validate_method_arity(method, *needed_args):
    """
    Tests if the decorated method has a sufficient number of parameters.
    
    :param method: The method to be tested
    :param needed_args: The name (for description only) of the needed arguments,
                        without "self".
    :return: Nothing
    :raise TypeError: Invalid number of parameter
    """
    nb_needed_args = len(needed_args) + 1

    # Test the number of parameters
    argspec = inspect.getargspec(method)
    method_args = argspec.args

    if len(method_args) == 0:
        # No argument at all
        raise TypeError("Decorated method {0} must have at least the 'self' "
                        "parameter".format(get_method_description(method)))

    if argspec.varargs is not None:
        # Variable arguments
        if len(method_args) != 1 or method_args[0] != "self":
            # Other arguments detected
            raise TypeError("When using '*args', the decorated {0} method must "
                            "only accept the 'self' argument".format(
                                get_method_description(method)))

    elif len(method_args) != nb_needed_args or method_args[0] != 'self':
        # "Normal" arguments
        raise TypeError("The decorated method {0} must accept exactly {1} "
                        "parameters : (self, {2})".format(
                                get_method_description(method), nb_needed_args,
                                ", ".join(needed_args)))

# ------------------------------------------------------------------------------

def _get_factory_context(cls):
    """
    Retrieves the factory context object associated to a factory. Creates it
    if needed
    
    :param cls: The factory class
    :return: The factory class context
    """
    context = getattr(cls, constants.IPOPO_FACTORY_CONTEXT_DATA, None)

    if context is None:
        # Class not yet manipulated
        context = FactoryContext()

    else:
        if is_from_parent(cls, constants.IPOPO_FACTORY_CONTEXT_DATA):
            # The context comes from a parent, copy it using a temporary
            # dictionary form
            if isinstance(context, dict):
                context = FactoryContext.from_dictionary_form(context)

            context = context.copy()

            # Clear the values that must not be inherited:
            # * Provided services
            del context.provides[:]

        # We have a context of our own, make sure we have a FactoryContext
        if isinstance(context, dict):
            # Already manipulated and stored class
            context = FactoryContext.from_dictionary_form(context)

    setattr(cls, constants.IPOPO_FACTORY_CONTEXT_DATA, context)
    return context


def _ipopo_setup_callback(cls, context):
    """
    Sets up the class _callback dictionary
    
    :param cls: The class to handle
    :param context: The factory class context
    """
    assert inspect.isclass(cls)
    assert isinstance(context, FactoryContext)

    if context.callbacks is not None:
        callbacks = context.callbacks.copy()

    else:
        callbacks = {}

    functions = inspect.getmembers(cls, inspect.isroutine)

    for name, function in functions:

        if not hasattr(function, constants.IPOPO_METHOD_CALLBACKS):
            # No attribute, get the next member
            continue

        method_callbacks = getattr(function, constants.IPOPO_METHOD_CALLBACKS)

        if not isinstance(method_callbacks, list):
            # Invalid content
            _logger.warning("Invalid attribute %s in %s", \
                            constants.IPOPO_METHOD_CALLBACKS, name)
            continue

        # Keeping it allows inheritance : by removing it, only the first
        # child will see the attribute -> Don't remove it

        # Store the call backs
        for _callback in method_callbacks:
            if _callback in callbacks and \
            not is_from_parent(cls, callbacks[_callback].__name__):
                _logger.warning("Redefining the callback %s in '%s'. " \
                                "Previous callback : '%s' (%s). " \
                                "New callback : %s", _callback, name,
                                callbacks[_callback].__name__,
                                callbacks[_callback], function)

            callbacks[_callback] = function

    # Update the factory context
    context.callbacks.clear()
    context.callbacks.update(callbacks)


def _ipopo_setup_field_callback(cls, context):
    """
    Sets up the class _field_callback dictionary
    
    :param cls: The class to handle
    :param context: The factory class context
    """
    assert inspect.isclass(cls)
    assert isinstance(context, FactoryContext)

    if context.field_callbacks is not None:
        callbacks = context.field_callbacks.copy()

    else:
        callbacks = {}

    functions = inspect.getmembers(cls, inspect.isroutine)

    for name, function in functions:
        if not hasattr(function, constants.IPOPO_METHOD_FIELD_CALLBACKS):
            # No attribute, get the next member
            continue

        method_callbacks = getattr(function,
                                   constants.IPOPO_METHOD_FIELD_CALLBACKS)
        if not isinstance(method_callbacks, list):
            # Invalid content
            _logger.warning("Invalid attribute %s in %s", \
                            constants.IPOPO_METHOD_FIELD_CALLBACKS, name)
            continue

        # Keeping it allows inheritance : by removing it, only the first
        # child will see the attribute -> Don't remove it

        # Store the call backs
        for kind, field in method_callbacks:
            fields_cbs = callbacks.setdefault(field, {})

            if kind in fields_cbs and \
            not is_from_parent(cls, fields_cbs[kind].__name__):
                _logger.warning("Redefining the callback %s in '%s'. " \
                                "Previous callback : '%s' (%s). " \
                                "New callback : %s", kind, name,
                                fields_cbs[kind].__name__,
                                fields_cbs[kind], function)

            fields_cbs[kind] = function

    # Update the factory context
    context.field_callbacks.clear()
    context.field_callbacks.update(callbacks)

# ------------------------------------------------------------------------------

def _append_object_entry(obj, list_name, entry):
    """
    Appends the given entry in the given object list.
    Creates the list field if needed.
    
    :param obj: The object that contains the list
    :param list_name: The name of the list member in *obj*
    :param entry: The entry to be added to the list
    :raise ValueError: Invalid attribute content
    """
    # Get the list
    obj_list = getattr(obj, list_name, None)
    if obj_list is None:
        # We'll have to create it
        obj_list = []
        setattr(obj, list_name, obj_list)

    assert isinstance(obj_list, list)

    # Set up the property, if needed
    if entry not in obj_list:
        obj_list.append(entry)

# ------------------------------------------------------------------------------

class Holder(object):
    """
    Simple class that holds a value
    """
    def __init__(self, value):
        """
        Sets up the holder instance
        """
        self.value = value


def _ipopo_class_field_property(name, value, methods_prefix):
    """
    Sets up an iPOPO field property, using Python property() capabilities
    
    :param name: The property name
    :param value: The property default value
    :param methods_prefix: The common prefix of the getter and setter injected
                           methods
    :return: A generated Python property()
    """
    # The property lock
    lock = threading.RLock()

    # Prepare the methods names
    getter_name = "{0}{1}".format(methods_prefix, constants.IPOPO_GETTER_SUFFIX)
    setter_name = "{0}{1}".format(methods_prefix, constants.IPOPO_SETTER_SUFFIX)

    local_holder = Holder(value)

    def get_value(self):
        """
        Retrieves the property value, from the iPOPO dictionaries
        """
        getter = getattr(self, getter_name, None)
        if getter is not None:
            # Use the component getter
            with lock:
                return getter(self, name)

        else:
            # Use the local holder
            return local_holder.value


    def set_value(self, new_value):
        """
        Sets the property value and trigger an update event
        
        :param new_value: The new property value
        """
        setter = getattr(self, setter_name, None)
        if setter is not None:
            # Use the component setter
            with lock:
                setter(self, name, new_value)

        else:
            # Change the local holder
            local_holder.value = new_value

    return property(get_value, set_value)

# ------------------------------------------------------------------------------

class Instantiate:
    """
    Decorator that sets up a future instance of a component
    """
    def __init__(self, name, properties=None):
        """
        Sets up the decorator
        
        :param name: Instance name
        :param properties: Instance properties
        """
        if not is_string(name):
            raise TypeError("Instance name must be a string")

        if properties is not None and not isinstance(properties, dict):
            raise TypeError("Instance properties must be a dictionary or None")

        name = name.strip()
        if not name:
            raise ValueError("Invalid instance name '{0}'".format(name))

        self.__name = name
        self.__properties = properties


    def __call__(self, factory_class):
        """
        Sets up and registers the instances descriptions
        
        :param factory_class: The factory class to instantiate
        :return: The decorated factory class
        :raise TypeError: The given object is not a class
        """
        if not inspect.isclass(factory_class):
            raise TypeError("@ComponentFactory can decorate only classes, " \
                            "not '{0}'".format(type(factory_class).__name__))

        instances = getattr(factory_class, constants.IPOPO_INSTANCES, None)

        if instances is None or \
            is_from_parent(factory_class, constants.IPOPO_INSTANCES):
            # No instances for this particular class
            instances = {}
            setattr(factory_class, constants.IPOPO_INSTANCES, instances)

        if self.__name not in instances:
            instances[self.__name] = self.__properties

        else:
            _logger.warn("Component '%s' defined twice, new definition ignored",
                         self.__name)

        return factory_class

# ------------------------------------------------------------------------------

class ComponentFactory:
    """
    Decorator that sets up a component factory class
    """

    NON_INHERITABLE_FIELDS = (constants.IPOPO_INSTANCES,)
    """
    Non inheritable fields, i.e. to remove during the manipulation of child
    classes, if needed
    """

    def __init__(self, name=None):
        """
        Sets up the decorator

        :param name: Name of the component factory
        """
        self.__factory_name = name


    def __call__(self, factory_class):
        """
        Sets up and registers the factory class

        :param factory_class: The class to decorate
        :return: The decorated class
        :raise TypeError: The given object is not a class
        """
        if not inspect.isclass(factory_class):
            raise TypeError("@ComponentFactory can decorate only classes, " \
                            "not '{0}'".format(type(factory_class).__name__))

        # Get the factory context
        context = _get_factory_context(factory_class)

        # Set the factory name
        if not self.__factory_name:
            self.__factory_name = factory_class.__name__ + "Factory"

        context.name = self.__factory_name

        # Find callbacks
        _ipopo_setup_callback(factory_class, context)
        _ipopo_setup_field_callback(factory_class, context)

        # Clean up inherited fields, to avoid weird behavior
        for field in ComponentFactory.NON_INHERITABLE_FIELDS:
            if is_from_parent(factory_class, field):
                # Set inherited fields to None
                setattr(factory_class, field, None)

        # Add the factory context field (set it to None)
        setattr(factory_class, constants.IPOPO_FACTORY_CONTEXT, None)

        # Store a dictionary form of the factory context in the class
        # -> Avoids "class version" problems
        setattr(factory_class, constants.IPOPO_FACTORY_CONTEXT_DATA, \
                context.to_dictionary_form())

        # Inject the properties getter and setter if needed
        if len(context.properties_fields) > 0:
            setattr(factory_class, constants.IPOPO_PROPERTY_PREFIX \
                    + constants.IPOPO_GETTER_SUFFIX, None)
            setattr(factory_class, constants.IPOPO_PROPERTY_PREFIX \
                    + constants.IPOPO_SETTER_SUFFIX, None)

        return factory_class

# ------------------------------------------------------------------------------

class Property:
    """
    @Property decorator
    
    Defines a component property.
    """
    def __init__(self, field=None, name=None, value=None):
        """
        Sets up the property
        
        :param field: The property field in the class (can't be None nor empty)
        :param name: The property name (if None, this will be the field name)
        :param value: The property value
        :raise TypeError: Invalid argument type
        :raise ValueError: If the name or the name is None or empty
        """
        # Field validity test
        if not is_string(field):
            raise TypeError("Field name must be a string")

        field = field.strip()
        if not field or ' ' in field:
            raise ValueError("Empty or invalid property field name '{0}'"
                             .format(field))

        # Name validity test
        if name is not None:
            if not is_string(name):
                raise TypeError("Property name must be a string")

            name = name.strip()

        if not name:
            # No name given: use the field name
            name = field

        self.__field = field
        self.__name = name
        self.__value = value


    def __call__(self, clazz):
        """
        Adds the property to the class iPOPO properties field.
        Creates the field if needed.
        
        :param clazz: The class to decorate
        :return: The decorated class
        :raise TypeError: If *clazz* is not a type
        """
        if not inspect.isclass(clazz):
            raise TypeError("@Property can decorate only classes, not '{0}'" \
                            .format(type(clazz).__name__))

        # Get the factory context
        context = _get_factory_context(clazz)

        # Set up the property in the class
        context.properties[self.__name] = self.__value

        # Associate the field to the property name
        context.properties_fields[self.__field] = self.__name

        # Inject a property in the class. The property will call an instance
        # level getter / setter, injected by iPOPO after the instance creation
        setattr(clazz, self.__field, \
                _ipopo_class_field_property(self.__name, self.__value,
                                            constants.IPOPO_PROPERTY_PREFIX))

        return clazz

# ------------------------------------------------------------------------------

def _get_specifications(specifications):
    """
    Computes the list of strings corresponding to the given specifications
    
    :param specifications: A string, a class or a list of specifications
    :return: A list of strings
    :raise ValueError: Invalid specification found
    """
    if not specifications:
        raise ValueError("No specifications given")

    if inspect.isclass(specifications):
        # Get the name of the class
        return [specifications.__name__]

    elif is_string(specifications):
        # Specification name
        specifications = specifications.strip()
        if not specifications:
            raise ValueError("Empty specification given")

        return [specifications]

    elif isinstance(specifications, (list, tuple)):
        # List given: normalize its content
        results = []
        for specification in specifications:
            results.extend(_get_specifications(specification))

        return results

    else:
        raise ValueError("Unhandled specifications type : {0}" \
                         .format(type(specifications).__name__))


class Provides:
    """
    @Provides decorator
    
    Defines an interface exported by a component.
    """
    def __init__(self, specifications=None, controller=None):
        """
        Sets up a provided service.
        A service controller can be defined to enable or disable the service.
        
        :param specifications: A list of provided interface(s) name(s)
                               (can't be empty)
        :param controller: Name of the service controller class field (optional)
        :raise ValueError: If the specifications are invalid
        """
        if controller is not None:
            if not is_string(controller):
                raise ValueError("Controller name must be a string")

            controller = controller.strip()
            if not controller:
                # Empty controller name
                _logger.warning("Empty controller name given")
                controller = None

            elif ' ' in controller:
                raise ValueError("Controller name contains spaces")

        self.__specifications = _get_specifications(specifications)
        self.__controller = controller


    def __call__(self, clazz):
        """
        Adds the provided service information to the class context iPOPO field.
        Creates the field if needed.
        
        :param clazz: The class to decorate
        :return: The decorated class
        :raise TypeError: If *clazz* is not a type
        """

        if not inspect.isclass(clazz):
            raise TypeError("@Provides can decorate only classes, not '{0}'" \
                            .format(type(clazz).__name__))

        # Get the factory context
        context = _get_factory_context(clazz)

        # Avoid duplicates (but keep the order)
        filtered_specs = []
        for spec in self.__specifications:
            if spec not in filtered_specs:
                filtered_specs.append(spec)

        # Store the service information
        context.provides.append((filtered_specs, self.__controller))

        if self.__controller:
            # Inject a property in the class. The property will call an instance
            # level getter / setter, injected by iPOPO after the instance creation
            setattr(clazz, self.__controller,
                    _ipopo_class_field_property(self.__controller, True,
                                            constants.IPOPO_CONTROLLER_PREFIX))

            # Inject the future controller methods
            setattr(clazz, constants.IPOPO_CONTROLLER_PREFIX \
                    + constants.IPOPO_GETTER_SUFFIX, None)
            setattr(clazz, constants.IPOPO_CONTROLLER_PREFIX \
                    + constants.IPOPO_SETTER_SUFFIX, None)

        return clazz

# ------------------------------------------------------------------------------

class Requires:
    """
    @Requires decorator
    
    Defines a required component
    """
    def __init__(self, field="", specification="", aggregate=False, \
                 optional=False, spec_filter=None):
        """
        Sets up the requirement
        
        :param field: The injected field
        :param specification: The injected service specification
        :param aggregate: If true, injects a list
        :param optional: If true, this injection is optional
        :param spec_filter: An LDAP query to filter injected services upon their
                            properties
        :raise TypeError: A parameter has an invalid type
        :raise ValueError: An error occurred while parsing the filter or an
                           argument is incorrect
        """
        if not field:
            raise ValueError("Empty field name.")

        if not is_string(field):
            raise TypeError("The field name must be a string, not {0}" \
                            .format(type(field).__name__))

        if ' ' in field:
            raise ValueError("Field name can't contain spaces.")

        self.__field = field
        self.__requirement = Requirement(_get_specifications(specification),
                                         aggregate, optional, spec_filter)

    def __call__(self, clazz):
        """
        Adds the requirement to the class iPOPO field
        
        :param clazz: The class to decorate
        :return: The decorated class
        :raise TypeError: If *clazz* is not a type
        """
        if not inspect.isclass(clazz):
            raise TypeError("@Provides can decorate only classes, not '{0}'" \
                            .format(type(clazz).__name__))

        # Set up the property in the class
        context = _get_factory_context(clazz)
        context.requirements[self.__field] = self.__requirement

        # Inject the field
        setattr(clazz, self.__field, None)

        return clazz

# ------------------------------------------------------------------------------

class BindField(object):
    """
    BindField callback decorator, called when a component is bound to a
    dependency, injected in the given field.
    
    The decorated method must have the following prototype :
    
    .. python::
       def bind_method(self, field, service, service_reference):
           '''
           Method called when a service is bound to the component
           
           :param field: Field wherein the dependency is injected
           :param service: The injected service instance.
           :param service_reference: The injected service ServiceReference
           '''
           # ...
    
    If the service is a required one, the bind callback is called **before** the
    component is validated.
    The bind field callback is called **after** the global bind method.
    
    The service reference can be stored *if its reference is deleted on unbind*.
    
    Exceptions raised by a bind callback are ignored.
    """
    def __init__(self, field):
        """
        Sets up the decorator
        
        :param field: Field associated to the binding
        """
        self._field = field


    def __call__(self, method):
        """
        Updates the "field callback" list for this method
        
        :param method: Method to decorate
        :return: Decorated method
        :raise TypeError: The decorated element is not a valid function
        """
        if not inspect.isroutine(method):
            raise TypeError("@BindField can only be applied on functions")

        # Tests the number of parameters
        validate_method_arity(method, "field", "service", "service_reference")

        _append_object_entry(method, constants.IPOPO_METHOD_FIELD_CALLBACKS,
                             (constants.IPOPO_CALLBACK_BIND_FIELD,
                              self._field))
        return method


class UpdateField(object):
    """
    UpdateField callback decorator, called when a component dependency property
    has been modified.
    
    The decorated method must have the following prototype :
    
    .. python::
       def update_method(self, service, service_reference, old_properties):
           '''
           Method called when a service is bound to the component
           
           :param service: The injected service instance.
           :param service_reference: The injected service ServiceReference
           :param old_properties: Previous service properties
           '''
           # ...
    
    Exceptions raised by an update callback are ignored.
    """
    def __init__(self, field):
        """
        Sets up the decorator
        
        :param field: Field associated to the binding
        """
        self._field = field


    def __call__(self, method):
        """
        Updates the "field callback" list for this method
        
        :param method: Method to decorate
        :return: Decorated method
        :raise TypeError: The decorated element is not a valid function
        """
        if not inspect.isroutine(method):
            raise TypeError("@UnbindField can only be applied on functions")

        # Tests the number of parameters
        validate_method_arity(method, "field", "service", "service_reference",
                              "old_properties")

        _append_object_entry(method, constants.IPOPO_METHOD_FIELD_CALLBACKS,
                             (constants.IPOPO_CALLBACK_UPDATE_FIELD,
                              self._field))
        return method


class UnbindField(object):
    """
    UnbindField callback decorator, called when a component is unbound to a
    dependency, removed from the given field.
    
    The decorated method must have the following prototype :
    
    .. python::
       def unbind_method(self, field, service, service_reference):
           '''
           Method called when a service is bound to the component
           
           :param field: Field wherein the dependency is injected
           :param service: The injected service instance.
           :param service_reference: The injected service ServiceReference
           '''
           # ...
    
    If the service is a required one, the unbind callback is called **after**
    the component has been invalidated.
    The unbind field callback is called **before** the global unbind method.
    
    Exceptions raised by an unbind callback are ignored.
    """
    def __init__(self, field):
        """
        Sets up the decorator
        
        :param field: Field associated to the binding
        """
        self._field = field


    def __call__(self, method):
        """
        Updates the "field callback" list for this method
        
        :param method: Method to decorate
        :return: Decorated method
        :raise TypeError: The decorated element is not a valid function
        """
        if not inspect.isroutine(method):
            raise TypeError("@UnbindField can only be applied on functions")

        # Tests the number of parameters
        validate_method_arity(method, "field", "service", "service_reference")

        _append_object_entry(method, constants.IPOPO_METHOD_FIELD_CALLBACKS,
                             (constants.IPOPO_CALLBACK_UNBIND_FIELD,
                              self._field))
        return method

# ------------------------------------------------------------------------------

def Bind(method):
    """
    Bind callback decorator, called when a component is bound to a dependency.
    
    The decorated method must have the following prototype :
    
    .. python::
       def bind_method(self, service, service_reference):
           '''
           Method called when a service is bound to the component
           
           :param service: The injected service instance.
           :param service_reference: The injected service ServiceReference
           '''
           # ...
    
    If the service is a required one, the bind callback is called **before** the
    component is validated.
    
    The service reference can be stored *if its reference is deleted on unbind*.
    
    Exceptions raised by a bind callback are ignored.
    
    :param method: The decorated method
    :raise TypeError: The decorated element is not a valid function
    """
    if not inspect.isroutine(method):
        raise TypeError("@Bind can only be applied on functions")

    # Tests the number of parameters
    validate_method_arity(method, "service", "service_reference")

    _append_object_entry(method, constants.IPOPO_METHOD_CALLBACKS, \
                         constants.IPOPO_CALLBACK_BIND)
    return method


def Update(method):
    """
    Update callback decorator, called when a component dependency property has
    been modified.
    
    The decorated method must have the following prototype :
    
    .. python::
       def update_method(self, service, service_reference, old_properties):
           '''
           Method called when a service is bound to the component
           
           :param service: The injected service instance.
           :param service_reference: The injected service ServiceReference
           :param old_properties: Previous service properties
           '''
           # ...
    
    Exceptions raised by an update callback are ignored.
    
    :param method: The decorated method
    :raise TypeError: The decorated element is not a valid function
    """
    if type(method) is not types.FunctionType:
        raise TypeError("@Update can only be applied on functions")

    # Tests the number of parameters
    validate_method_arity(method, "service", "service_reference",
                          "old_properties")

    _append_object_entry(method, constants.IPOPO_METHOD_CALLBACKS, \
                         constants.IPOPO_CALLBACK_UPDATE)
    return method


def Unbind(method):
    """
    Unbind callback decorator, called when a component dependency is unbound.
    
    The decorated method must have the following prototype :
    
    .. python::
       def unbind_method(self, service, service_reference):
           '''
           Method called when a service is bound to the component
           
           :param service: The injected service instance.
           :param service_reference: The injected service ServiceReference
           '''
           # ...
    
    If the service is a required one, the unbind callback is called **after**
    the component has been invalidated.
    
    Exceptions raised by an unbind callback are ignored.
    
    :param method: The decorated method
    :raise TypeError: The decorated element is not a valid function
    """
    if type(method) is not types.FunctionType:
        raise TypeError("@Unbind can only be applied on functions")

    # Tests the number of parameters
    validate_method_arity(method, "service", "service_reference")

    _append_object_entry(method, constants.IPOPO_METHOD_CALLBACKS, \
                         constants.IPOPO_CALLBACK_UNBIND)
    return method


def Validate(method):
    """
    Validation callback decorator, called when a component becomes valid,
    i.e. if all of its required dependencies has been injected.
    
    The decorated method must have the following prototype :
    
    .. python::
       def validation_method(self, bundle_context):
           '''
           Method called when the component is validated
           
           :param bundle_context: The component's bundle context
           '''
           # ...
    
    If the validation callback raises an exception, the component is considered
    not validated.
    
    If the component provides a service, the validation method is called before
    the provided service is registered to the framework.
    
    :param method: The decorated method
    :raise TypeError: The decorated element is not a valid function
    """
    if type(method) is not types.FunctionType:
        raise TypeError("@Validate can only be applied on functions")

    # Tests the number of parameters
    validate_method_arity(method, "bundle_context")

    _append_object_entry(method, constants.IPOPO_METHOD_CALLBACKS, \
                         constants.IPOPO_CALLBACK_VALIDATE)
    return method


def Invalidate(method):
    """
    Invalidation callback decorator, called when a component becomes invalid,
    i.e. if one of its required dependencies disappeared
    
    The decorated method must have the following prototype :
    
    .. python::
       def invalidation_method(self, bundle_context):
           '''
           Method called when the component is invalidated
           
           :param bundle_context: The component's bundle context
           '''
           # ...
    
    Exceptions raised by an invalidation callback are ignored.
    
    If the component provides a service, the invalidation method is called after
    the provided service has been unregistered to the framework.
    
    :param method: The decorated method
    :raise TypeError: The decorated element is not a function
    """

    if type(method) is not types.FunctionType:
        raise TypeError("@Invalidate can only be applied on functions")

    # Tests the number of parameters
    validate_method_arity(method, "bundle_context")

    _append_object_entry(method, constants.IPOPO_METHOD_CALLBACKS, \
                         constants.IPOPO_CALLBACK_INVALIDATE)
    return method




#!/usr/bin/env python
# -- Content-Encoding: UTF-8 --
"""
EventAdmin shell commands

Provides commands to the Pelix shell to work with the EventAdmin service

:author: Thomas Calmant
:copyright: Copyright 2013, isandlaTech
:license: GPLv3
:version: 0.1
:status: Alpha

..

    This file is part of iPOPO.

    iPOPO is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    iPOPO is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with iPOPO. If not, see <http://www.gnu.org/licenses/>.
"""

# Module version
__version_info__ = (0, 1, 1)
__version__ = ".".join(map(str, __version_info__))

# Documentation strings format
__docformat__ = "restructuredtext en"

# -----------------------------------------------------------------------------

# Shell constants
from pelix.shell import SHELL_COMMAND_SPEC

# iPOPO Decorators
from pelix.ipopo.decorators import ComponentFactory, Requires, Provides, \
    Instantiate
import pelix.services

# ------------------------------------------------------------------------------

@ComponentFactory("eventadmin-shell-commands-factory")
@Requires("_events", pelix.services.SERVICE_EVENT_ADMIN)
@Provides(SHELL_COMMAND_SPEC)
@Instantiate("eventadmin-shell-commands")
class EventAdminCommands(object):
    """
    EventAdmin shell commands
    """
    def __init__(self):
        """
        Sets up members
        """
        # Injected services
        self._events = None


    def get_namespace(self):
        """
        Retrieves the name space of this command handler
        """
        return "event"


    def get_methods(self):
        """
        Retrieves the list of tuples (command, method) for this command handler
        """
        return [("send", self.send),
                ("post", self.post)]


    def get_methods_names(self):
        """
        Retrieves the list of tuples (command, method name) for this command
        handler.
        """
        return [(command, method.__name__)
                for command, method in self.get_methods()]


    def send(self, io_handler, topic, **kwargs):
        """
        send <topic> [<property=value>] - Sends an event (blocking)
        """
        self._events.send(topic, kwargs)


    def post(self, io_handler, topic, **kwargs):
        """
        post <topic> [<property=value>] - Posts an event (asynchronous)
        """
        self._events.post(topic, kwargs)



#!/usr/bin/env python
# -- Content-Encoding: UTF-8 --
"""
Pelix shell package

:author: Thomas Calmant
:copyright: Copyright 2013, isandlaTech
:license: GPLv3
:version: 0.1
:status: Alpha

..

    This file is part of iPOPO.

    iPOPO is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.
    
    iPOPO is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
    
    You should have received a copy of the GNU General Public License
    along with iPOPO. If not, see <http://www.gnu.org/licenses/>.
"""

# Module version
__version_info__ = (0, 1, 0)
__version__ = ".".join(map(str, __version_info__))

# Documentation strings format
__docformat__ = "restructuredtext en"

# ------------------------------------------------------------------------------

SHELL_SERVICE_SPEC = "pelix.shell"
"""
Core shell service:

* register_command(ns, command, method): Registers a command in the given name
  space, executing the given method
* unregister(namespace, command): If command is given, unregisters it, else
  unregisters the whole given namespace
* execute(cmdline, stdin, stdout): Executes the given command line with the
  given input and output streams
"""

SHELL_COMMAND_SPEC = "pelix.shell.command"
"""
Shell commands service, for auto-registration (white board pattern).

* get_namespace(): returns the name space of the handler
* get_methods(): returns a command name -> method dictionary
* get_methods_names(): returns a command name -> method name dictionary
"""

SHELL_UTILS_SERVICE_SPEC = "pelix.shell.utilities"
"""
Shell utility service:

* make_table(headers, lines): to make ASCII arrays
* bundlestate_to_str(state): to get the string representation of a bundle state
"""

REMOTE_SHELL_SPEC = "pelix.shell.remote"
"""
Remote shell service

* get_access(): returns the (host, port) tuple where the remote shell is waiting
  clients. 
"""

# ------------------------------------------------------------------------------



#!/usr/bin/env python
# -- Content-Encoding: UTF-8 --
"""
Pelix interactive shell

Provides a console interface for the Pelix shell, based on readline when
available.

:author: Thomas Calmant
:copyright: Copyright 2013, isandlaTech
:license: GPLv3
:version: 0.1.1
:status: Alpha

..

    This file is part of iPOPO.

    iPOPO is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    iPOPO is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with iPOPO. If not, see <http://www.gnu.org/licenses/>.
"""

# Module version
__version_info__ = (0, 1, 1)
__version__ = ".".join(map(str, __version_info__))

# Documentation strings format
__docformat__ = "restructuredtext en"

# ------------------------------------------------------------------------------

# Shell constants
from pelix.shell import SHELL_SERVICE_SPEC
import pelix.framework as pelix

# Standard library
import logging
import sys
import threading

# ------------------------------------------------------------------------------

_logger = logging.getLogger(__name__)

# ------------------------------------------------------------------------------

try:
    # Set up readline if available
    import readline
    readline.parse_and_bind('tab: complete')
    readline.set_completer(None)

except ImportError:
    # Readline is missing, not critical
    readline = None

# Before Python 3, input() was raw_input()
if sys.version_info[0] < 3:
    safe_input = raw_input

else:
    safe_input = input

# ------------------------------------------------------------------------------

class InteractiveShell(object):
    """
    The interactive shell handler
    """
    def __init__(self, context):
        """
        Sets up the members
        
        :param context: The bundle context
        """
        self._context = context
        self._shell_ref = None
        self._shell = None

        # Read line cache
        self._readline_matches = []

        # Rendez-vous events
        self._lock = threading.RLock()
        self._stop_event = threading.Event()
        self._shell_event = threading.Event()

        # Try to find a shell service
        self.search_shell()

        # Register as a service listener
        ldap_filter = '({0}={1})'.format(pelix.OBJECTCLASS, SHELL_SERVICE_SPEC)
        self._context.add_service_listener(self, ldap_filter)


    def loop_input(self, on_quit=None):
        """
        Reads the standard input until the shell session is stopped
        
        :param on_quit: A call back method, called without argument when the
                        shell session has ended
        """
        try:
            first_prompt = True

            while not self._stop_event.is_set():
                # Wait for the shell to be there
                # Before Python 2.7, wait() doesn't return a result
                if self._shell_event.wait(.2) or self._shell_event.is_set():
                    # Shell present
                    if first_prompt:
                        # Show the banner on first prompt
                        print(self._shell.get_banner())
                        first_prompt = False

                    # Read the line
                    line = safe_input(self._shell.get_ps1())

                    with self._lock:
                        if self._shell_event.is_set():
                            # Execute it
                            self._shell.execute(line, sys.stdin, sys.stdout)

                        elif not self._stop_event.is_set():
                            # Shell service lost while not stopping
                            print('Shell service lost.')

        except (EOFError, KeyboardInterrupt, SystemExit):
            # Input closed or keyboard interruption
            self._stop_event.set()

        print('Bye !')
        if on_quit is not None:
            # Call a handler if needed
            on_quit()


    def readline_completer(self, text, state):
        """
        A completer for the readline library
        """
        if state == 0:
            if '.' in text:
                # A name space is given
                namespace, prefix = text.split('.', 2)
                commands = self._shell.get_commands(namespace)

                # Filter methods according to the prefix
                self._readline_matches = ['{0}.{1}'.format(namespace, command)
                                          for command in commands
                                          if command.startswith(prefix)]

            else:
                # Complete with name space names or default commands
                prefix = text
                possibilities = self._shell.get_commands(None)
                possibilities.extend(['{0}.'.format(namespace)
                                      for namespace in self._shell.get_namespaces()])

                # Filter methods according to the prefix
                self._readline_matches = [entry for entry in possibilities
                                          if entry.startswith(prefix)]

            if not self._readline_matches:
                return None

            # Return the first possibility
            return self._readline_matches[0]

        elif state < len(self._readline_matches):
            # Next try
            return self._readline_matches[state]


    def search_shell(self):
        """
        Looks for a shell service
        """
        with self._lock:
            if self._shell is not None:
                # A shell is already there
                return

            reference = self._context.get_service_reference(SHELL_SERVICE_SPEC)
            if reference is not None:
                self.set_shell(reference)


    def service_changed(self, event):
        """
        Called by Pelix when an events changes
        """
        kind = event.get_kind()
        reference = event.get_service_reference()

        if kind in (pelix.ServiceEvent.REGISTERED,
                    pelix.ServiceEvent.MODIFIED):
            # A service matches our filter
            self.set_shell(reference)

        else:
            with self._lock:
                # Service is not matching our filter anymore
                self.clear_shell()

                # Request for a new binding
                self.search_shell()


    def set_shell(self, svc_ref):
        """
        Binds the given shell service.
        
        :param svc_ref: A service reference
        """
        if svc_ref is None:
            return

        with self._lock:
            # Get the service
            self._shell_ref = svc_ref
            self._shell = self._context.get_service(self._shell_ref)

            # Set the readline completer
            if readline is not None:
                readline.set_completer(self.readline_completer)

            # Set the flag
            self._shell_event.set()


    def clear_shell(self):
        """
        Unbinds the active shell service
        """
        with self._lock:
            # Clear the flag
            self._shell_event.clear()

            # Clear the readline completer
            if readline is not None:
                readline.set_completer(None)
                del self._readline_matches[:]

            if self._shell_ref is not None:
                # Release the service
                self._context.unget_service(self._shell_ref)

            self._shell_ref = None
            self._shell = None
            self._readline_matches = None


    def stop(self):
        """
        Clears all members
        """
        # Exit the loop
        with self._lock:
            self._stop_event.set()
            self._shell_event.clear()

        if self._context is not None:
            # Unregister from events
            self._context.remove_service_listener(self)

            # Release the shell
            self.clear_shell()
            self._context = None

# ------------------------------------------------------------------------------

class Activator(object):
    """
    The bundle activator
    """
    def __init__(self):
        """
        Sets up the members
        """
        self._context = None
        self._shell = None
        self._thread = None


    def start(self, context):
        """
        Bundle started
        """
        self._context = context
        self._shell = InteractiveShell(context)

        # Run the loop thread
        self._thread = threading.Thread(target=self._shell.loop_input,
                                        args=[self._quit])
        # Set the thread as a daemon, to let it be killed by the interpreter
        # once all other threads stopped.
        self._thread.daemon = True
        self._thread.start()


    def stop(self, context):
        """
        Bundle stopped
        """
        self._cleanup()
        self._context = None


    def _quit(self):
        """
        Called when the shell session has ended
        """
        # Clean up members
        self._cleanup()

        # Stop the framework
        if self._context is not None:
            self._context.get_bundle(0).stop()


    def _cleanup(self):
        """
        Cleans up the members
        """
        if self._shell is not None:
            # Stop the shell
            self._shell.stop()

        self._thread = None
        self._shell = None

# ------------------------------------------------------------------------------

def main():
    """
    Entry point
    """
    # Use the utility method to create, run and delete the framework
    pelix.create_framework(('pelix.ipopo.core', 'pelix.shell.core',
                            'pelix.shell.console', 'pelix.shell.ipopo'),
                           None, True, True, True)


if __name__ == '__main__':
    # Ran as a script:
    # Prepare the logger
    logging.basicConfig(level=logging.WARNING)

    # Run the entry point
    main()

else:
    # Imported as a module:
    # Creates the Pelix activator
    activator = Activator()



#!/usr/bin/env python
# -- Content-Encoding: UTF-8 --
"""
Pelix remote services package

:author: Thomas Calmant
:copyright: Copyright 2013, isandlaTech
:license: GPLv3
:version: 0.1
:status: Alpha

..

    This file is part of iPOPO.

    iPOPO is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.
    
    iPOPO is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
    
    You should have received a copy of the GNU General Public License
    along with iPOPO. If not, see <http://www.gnu.org/licenses/>.
"""

# Module version
__version_info__ = (0, 1, 0)
__version__ = ".".join(map(str, __version_info__))

# Documentation strings format
__docformat__ = "restructuredtext en"

#-------------------------------------------------------------------------------

# Pelix framework constants
import pelix.framework

#-------------------------------------------------------------------------------

SERVICE_DISPATCHER = "pelix.remote.dispatcher"
""" Remote call dispatcher """

SERVICE_DISPATCHER_SERVLET = "pelix.remote.dispatcher.servlet"
""" Servlet to access the content of the dispatcher """

SERVICE_REGISTRY = "pelix.remote.registry"
""" Registry of imported end points """

SERVICE_ENDPOINT_LISTENER = "pelix.remote.endpoint.listener"
"""
End point creation/deletion listeners, with listen.exported and/or
listen.imported properties.
"""

PROP_FRAMEWORK_UID = "pelix.remote.framework.uid"
"""
The UID of the framework that exports the service.
This service property is set by the discoverer, when it parses an end point
event packet.
"""

PROP_ENDPOINT_NAME = "endpoint.name"
""" Name of the end point of an exported service """

PREFIX_PROP_EXPORTED = "service.exported."
""" Prefix common to all export properties (for filtering) """

PROP_EXPORTED_CONFIGS = "{0}configs".format(PREFIX_PROP_EXPORTED)
""" Export configurations (xmlrpc, ...) (array of strings) """

PROP_EXPORTED_INTERFACES = "{0}interfaces".format(PREFIX_PROP_EXPORTED)
""" Exported specifications: must be an array of strings """

PROP_IMPORTED = "service.imported"
""" Flag indicating that the service has been imported """

PROP_IMPORTED_CONFIGS = "service.imported.configs"
""" Configurations of the imported service (mirror of PROP_EXPORTED_CONFIGS) """

PROP_IMPORTED_INTERFACES = "service.imported.interfaces"
""" Imported specifications (mirror of PROP_EXPORTED_INTERFACES) """

PROP_LISTEN_IMPORTED = "listen.imported"
"""
If set to True, the end point listener will be notified of imported end points
"""

PROP_LISTEN_EXPORTED = "listen.exported"
"""
If set to True, the end point listener will be notified of exported end points
"""

#-------------------------------------------------------------------------------

class RemoteServiceError(Exception):
    """
    Error while accessing a remote service entry
    """
    pass

#-------------------------------------------------------------------------------

class ExportEndpoint(object):
    """
    Represents an end point to access an exported service
    """
    def __init__(self, uid, kind, name, svc_ref, service, url):
        """
        Sets up the members
        
        :param uid: Unique identified of the end point
        :param kind: Kind of end point (xmlrpc, ...)
        :param name: Name of the end point
        :param svc_ref: ServiceReference of the exported service
        :param service: Instance of the exported service
        :param url: URL to access to the end point
        :raise ValueError: Invalid UID or the end point exports nothing
                           (all specifications have been filtered) 
        """
        if not uid:
            raise ValueError("Invalid GUID")

        # Given information
        self.__uid = uid
        self.__instance = service
        self.__reference = svc_ref
        self.__kind = kind
        self.__name = name
        self.__url = url

        # Exported specifications
        self.__exported_specs = []
        self.__compute_specifications()
        if not self.__exported_specs:
            raise ValueError("End point %s, %s, exports nothing",
                             self.__uid, self.__name)


    def __eq__(self, other):
        """
        Equality checked by UID
        """
        return self.__uid == other.uid


    def __ne__(self, other):
        """
        Inequality checked by UID
        """
        return self.__uid != other.uid


    def __compute_specifications(self):
        """
        Computes the list of exported specifications
        """
        specs = self.__reference.get_property(pelix.framework.OBJECTCLASS)
        exported_specs = self.__reference.get_property(PROP_EXPORTED_INTERFACES)

        if exported_specs and exported_specs != "*":
            # A set of specifications is exported, replace "objectClass"
            if isinstance(exported_specs, (list, tuple, set)):
                self.__exported_specs = [spec for spec in specs
                                         if spec in exported_specs]

        else:
            # Export everything
            self.__exported_specs = specs


    # Access to the service
    @property
    def instance(self):
        """
        Service instance
        """
        return self.__instance


    @property
    def reference(self):
        """
        Service reference
        """
        return self.__reference


    # End point properties
    @property
    def uid(self):
        """
        End point unique identifier
        """
        return self.__uid


    @property
    def kind(self):
        """
        Kind of end point
        """
        return self.__kind

    @property
    def name(self):
        """
        Name of the end point
        """
        return self.__name

    @property
    def specifications(self):
        """
        Returns the exported specifications
        """
        return self.__exported_specs

    @property
    def url(self):
        """
        URL to access the end point
        """
        return self.__url

#-------------------------------------------------------------------------------

class ImportEndpoint(object):
    """
    Represents an end point to access an imported service
    """
    def __init__(self, uid, kind, name, url, specifications, properties):
        """
        Sets up the members
        
        :param uid: Unique identified of the end point
        :param kind: Kind of end point (xmlrpc, ...)
        :param name: Name of the end point
        :param url: URL to access to the end point
        :param specifications: Specifications of the exported service
        :param properties: Properties of the service
        """
        self.__uid = uid
        self.__kind = kind
        self.__name = name
        self.__url = url
        self.__specifications = specifications
        self.__properties = properties.copy() if properties else {}

    # Access to the service informations
    @property
    def specifications(self):
        """
        Specifications of the service
        """
        return self.__specifications


    @property
    def properties(self):
        """
        Specifications of the service
        """
        return self.__properties


    @properties.setter
    def properties(self, properties):
        """
        Specifications of the service
        """
        self.__properties = properties


    # End point properties
    @property
    def uid(self):
        """
        End point unique identifier
        """
        return self.__uid

    @property
    def kind(self):
        """
        Kind of end point
        """
        return self.__kind

    @property
    def name(self):
        """
        Name of the end point
        """
        return self.__name

    @property
    def url(self):
        """
        URL to access the end point
        """
        return self.__url
